"""Constants for the calculation of nominal concrete cover according to NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020."""

from dataclasses import dataclass, field

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover._base_classes.nominal_cover_constants import (
    AbrasionClass,
    CastingSurface,
    NominalConcreteCoverConstantsBase,
)
from blueprints.type_alias import MM


@dataclass(frozen=True)
class NominalConcreteCoverConstants(NominalConcreteCoverConstantsBase):
    """Constants for the calculation of nominal concrete cover according to NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020."""

    CODE_PREFIX: str = field(default="NEN-", init=False)

    CODE_SUFFIX: str = field(default=":2005+A1:2015+NB:2016+A1:2020", init=False)

    # According to art. 4.4.1.2 (11) from NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020
    COVER_INCREASE_FOR_UNEVEN_SURFACE: MM = field(default=5, init=False)

    # According to art. 4.4.1.3 (1) from NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020
    DEFAULT_DELTA_C_DEV: MM = field(default_factory=int)

    # According to art. 4.4.1.2 (13) from NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020
    COVER_INCREASE_FOR_ABRASION_CLASS: dict[AbrasionClass, MM] = field(default_factory=dict)

    def __post_init__(self) -> None:
        """Post-initialization method to set default values."""
        if not self.COVER_INCREASE_FOR_ABRASION_CLASS:
            abrasion_class_cover_increase = {
                AbrasionClass.NA: 0,
                AbrasionClass.XM1: 0,  # k1 according to art. 4.4.1.2 (13)
                AbrasionClass.XM2: 0,  # k2 according to art. 4.4.1.2 (13)
                AbrasionClass.XM3: 0,  # k3 according to art. 4.4.1.2 (13)
            }
            object.__setattr__(self, "COVER_INCREASE_FOR_ABRASION_CLASS", abrasion_class_cover_increase)
        if not self.DEFAULT_DELTA_C_DEV:
            object.__setattr__(self, "DEFAULT_DELTA_C_DEV", 5)

    @staticmethod
    def minimum_cover_with_regard_to_casting_surface(c_min_dur: MM, casting_surface: CastingSurface) -> MM:
        """Calculate the minimum cover with regard to casting surface
        according to art. 4.4.1.3 (4) from NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020.
        """
        match casting_surface:
            case CastingSurface.PERMANENTLY_EXPOSED | CastingSurface.FORMWORK:
                return 0  # No additional requirements
            case CastingSurface.PREPARED_GROUND:
                return c_min_dur + 10  # k1 ≥ c_min,dur + 10
            case CastingSurface.DIRECTLY_AGAINST_SOIL:
                return c_min_dur + 50  # k2 ≥ c_min,dur + 50

    @staticmethod
    def minimum_cover_with_regard_to_casting_surface_latex(casting_surface: CastingSurface) -> str:
        """LateX representation of minimum cover with regard to casting surface
        according to art. 4.4.1.3 (4) from NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020.
        """
        match casting_surface:
            case CastingSurface.PERMANENTLY_EXPOSED | CastingSurface.FORMWORK:
                return f"0 (No additional requirements for {casting_surface.value})"
            case CastingSurface.PREPARED_GROUND:
                return f"k1 \\ge c_{{min,dur}} + 10 mm for {casting_surface.value}"
            case CastingSurface.DIRECTLY_AGAINST_SOIL:
                return f"k2 \\ge c_{{min,dur}} + 50 mm for {casting_surface.value}"



"""Formula 6.1 from NEN-EN 1993-1-1:2005: Chapter 6 - Ultimate limit state."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MPA
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot1ElasticVerification(Formula):
    r"""Class representing formula 6.1 for the elastic verification with the yield criterion."""

    label = "6.1"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        sigma_x_ed: MPA,
        sigma_z_ed: MPA,
        tau_ed: MPA,
        f_y: MPA,
        gamma_m0: DIMENSIONLESS,
    ) -> None:
        r"""Elastic verification with the yield criterion.

        NEN-EN 1993-1-1+C2:2016 art.6.2.1(5) - Formula (6.1)

        Parameters
        ----------
        sigma_x_ed : MPA
            [$\sigma_{x,\text{Ed}}$] Design value of the longitudinal stress at the point of consideration [$MPa$].
        sigma_z_ed : MPA
            [$\sigma_{z,\text{Ed}}$] Design value of the transverse stress at the point of consideration [$MPa$].
        tau_ed : MPA
            [$\tau_{\text{Ed}}$] Design value of the shear stress at the point of consideration [$MPa$].
        f_y : MPA
            [$f_y$] Yield strength of the material [$MPa$].
        gamma_m0 : DIMENSIONLESS
            [$\gamma_{M0}$] Partial safety factor for the material [dimensionless].
        """
        super().__init__()
        self.sigma_x_ed = sigma_x_ed
        self.sigma_z_ed = sigma_z_ed
        self.tau_ed = tau_ed
        self.f_y = f_y
        self.gamma_m0 = gamma_m0

    @staticmethod
    def _evaluate(
        sigma_x_ed: MPA,
        sigma_z_ed: MPA,
        tau_ed: MPA,
        f_y: MPA,
        gamma_m0: DIMENSIONLESS,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(sigma_x_ed=sigma_x_ed, sigma_z_ed=sigma_z_ed, tau_ed=tau_ed)
        raise_if_less_or_equal_to_zero(gamma_m0=gamma_m0, f_y=f_y)

        term1 = (sigma_x_ed / (f_y / gamma_m0)) ** 2
        term2 = (sigma_z_ed / (f_y / gamma_m0)) ** 2
        term3 = (sigma_x_ed / (f_y / gamma_m0)) * (sigma_z_ed / (f_y / gamma_m0))
        term4 = 3 * (tau_ed / (f_y / gamma_m0)) ** 2

        return term1 + term2 - term3 + term4 <= 1

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.1."""
        _equation: str = (
            r"\left( \frac{\sigma_{x,\text{Ed}}}{f_y / \gamma_{M0}} \right)^2 "
            r"+ \left( \frac{\sigma_{z,\text{Ed}}}{f_y / \gamma_{M0}} \right)^2 "
            r"- \left( \frac{\sigma_{x,\text{Ed}}}{f_y / \gamma_{M0}} \right) "
            r"\left( \frac{\sigma_{z,\text{Ed}}}{f_y / \gamma_{M0}} \right) "
            r"+ 3 \left( \frac{\tau_{\text{Ed}}}{f_y / \gamma_{M0}} \right)^2 \leq 1"
        )
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"\sigma_{x,\text{Ed}}": f"{self.sigma_x_ed:.{n}f}",
                r"\sigma_{z,\text{Ed}}": f"{self.sigma_z_ed:.{n}f}",
                r"\tau_{\text{Ed}}": f"{self.tau_ed:.{n}f}",
                r"f_y": f"{self.f_y:.{n}f}",
                r"\gamma_{M0}": f"{self.gamma_m0:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="\\to",
            unit="",
        )



"""Formula 12.2 from EN 1992-1-1:2004: Chapter 12 - Plain and Lightly Reinforced Concrete Structures."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form12Dot2PlainConcreteBendingResistance(Formula):
    r"""Class representing formula 12.2 for the calculation of the design bending resistance of plain concrete,
    :math:`N_{Rd}`.

    EN 1992-1-1:2004 art.12.6.1(3) - Formula (12.2)
    """

    label = "12.2"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        eta_f_cd_pl: MPA,
        b: MM,
        h_w: MM,
        e: MM,
    ) -> None:
        r"""[:math:`N_{Rd}`] Design bending resistance of plain concrete [:math:`N`].

        EN 1992-1-1:2004 art.12.6.1(3) - Formula (12.2)

        Parameters
        ----------
        eta_f_cd_pl : MPa
            [:math:`\eta f_{cd,pl}`] Effective design compressive strength [:math:`MPa`].
        b : mm
            [:math:`b`] Total width of the cross-section [:math:`mm`].
        h_w : mm
            [:math:`h_w`] Total height of the cross-section [:math:`mm`].
        e : mm
            [:math:`e`] Eccentricity of :math:`N_{Ed}` in the direction of :math:`h_w` [:math:`mm`].
        """
        super().__init__()
        self.eta_f_cd_pl = eta_f_cd_pl
        self.b = b
        self.h_w = h_w
        self.e = e

    @staticmethod
    def _evaluate(
        eta_f_cd_pl: MPA,
        b: MM,
        h_w: MM,
        e: MM,
    ) -> N:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            eta_f_cd_pl=eta_f_cd_pl,
            b=b,
            e=e,
        )
        raise_if_less_or_equal_to_zero(
            h_w=h_w,
        )
        return eta_f_cd_pl * b * h_w * (1 - 2 * e / h_w)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 12.2."""
        return LatexFormula(
            return_symbol=r"N_{Rd}",
            result=f"{self:.{n}f}",
            equation=r"\eta f_{cd,pl} \cdot b \cdot h_w \cdot \left(1 - \frac{2e}{h_w}\right)",
            numeric_equation=rf"{self.eta_f_cd_pl:.{n}f} \cdot {self.b:.{n}f} \cdot {self.h_w:.{n}f} "
            rf"\cdot \left(1 - \frac{{2 \cdot {self.e:.{n}f}}}{{{self.h_w:.{n}f}}}\right)",
            comparison_operator_label="=",
        )



"""Testing Formula 2.2 from NEN 9997-1+C2:2017: Chapter 2: Basis of geotechnical design."""

import pytest

from blueprints.codes.eurocode.nen_9997_1_c2_2017.chapter_2_basic_of_geotechnical_design.formula_2_2 import Form2Dot2DesignValueGeotechnicalParameter
from blueprints.validations import LessOrEqualToZeroError


class TestForm2Dot2DesignValueGeotechnicalParameter:
    """Validation for formula 2.2 from NEN 9997-1+C2:2017."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        assert Form2Dot2DesignValueGeotechnicalParameter(x_k=25.0, gamma_m=1.45) == pytest.approx(expected=17.241, abs=0.001)

    def test_raise_error_if_negative_or_zero_gamma(self) -> None:
        """Test that a LessOrEqualToZeroError is raised when a negative or zero value is passed for gamma_m."""
        with pytest.raises(LessOrEqualToZeroError):
            Form2Dot2DesignValueGeotechnicalParameter(x_k=25.0, gamma_m=-1.0)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            ("complete", r"X_d = \frac{X_{k}}{\gamma_M} = \frac{25.000}{1.450} = 17.241"),
            ("short", r"X_d = 17.241"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex representation of the formula."""
        # Object to test
        latex = Form2Dot2DesignValueGeotechnicalParameter(x_k=25.0, gamma_m=1.45).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Test the SHS enum."""

from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.shs import SHS


class TestSHS:
    """Tests for the SHS enum."""

    def test_enum_values(self) -> None:
        """Test that enum values are correctly defined."""
        assert SHS.SHS_40_2_6.value == ("SHS40x2.6", 40, 2.6, 3.9, 2.6)
        assert SHS.SHS_400_20.value == ("SHS400x20", 400, 20, 30, 20)

    def test_enum_membership(self) -> None:
        """Test that specific values are members of the enum."""
        assert "SHS40x2.6" in [e.value[0] for e in SHS]
        assert "SHS400x20" in [e.value[0] for e in SHS]

    def test_enum_uniqueness(self) -> None:
        """Test that all enum values are unique."""
        values = [e.value for e in SHS]
        assert len(values) == len(set(values))

    def test_enum_attributes(self) -> None:
        """Test that enum attributes are correctly assigned."""
        profile = SHS.SHS_40_2_6
        assert profile.alias == "SHS40x2.6"
        assert profile.total_width == 40
        assert profile.total_height == 40
        assert profile.thickness == 2.6
        assert profile.outer_radius == 3.9
        assert profile.inner_radius == 2.6



"""Testing formula 6.42 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_42 import Form6Dot42BetaCircular
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot42BetaCircular:
    """Validation for formula 6.42 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        d = 500.0
        diameter = 400.0
        e = 300.0

        # Object to test
        formula = Form6Dot42BetaCircular(d=d, diameter=diameter, e=e)

        # Expected result, manually calculated
        manually_calculated_result = 1.235619449

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("d", "diameter", "e"),
        [
            (-500.0, 400.0, 300.0),  # d is negative
            (500.0, -400.0, 300.0),  # D is negative
            (500.0, 400.0, -300.0),  # e is negative
            (0.0, 0.0, 300.0),  # denominator is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, d: float, diameter: float, e: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot42BetaCircular(d=d, diameter=diameter, e=e)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\beta = 1 + 0.6 \cdot \pi \cdot \frac{e}{D + 4 \cdot d} "
                r"= 1 + 0.6 \cdot \pi \cdot \frac{300.000}{400.000 + 4 \cdot 500.000} = 1.236 \ -",
            ),
            ("short", r"\beta = 1.236 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        d = 500.0
        diameter = 400.0
        e = 300.0

        # Object to test
        latex = Form6Dot42BetaCircular(d=d, diameter=diameter, e=e).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 7.19 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_7_serviceability_limit_state.formula_7_19 import Form7Dot19DistributionCoefficient
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm7Dot19DistributionCoefficient:
    """Validation for formula 7.19 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        beta = 0.5
        sigma_sr = 200.0
        sigma_s = 400.0

        # Object to test
        formula = Form7Dot19DistributionCoefficient(beta=beta, sigma_sr=sigma_sr, sigma_s=sigma_s)

        # Expected result, manually calculated
        manually_calculated_result = 0.875  # dimensionless

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("beta", "sigma_sr", "sigma_s"),
        [
            (-0.5, 200.0, 400.0),  # beta is negative
            (0.5, -200.0, 400.0),  # sigma_sr is negative
            (0.5, 200.0, -400.0),  # sigma_s is negative
            (0.5, 200.0, 0.0),  # sigma_s is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, beta: float, sigma_sr: float, sigma_s: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form7Dot19DistributionCoefficient(beta=beta, sigma_sr=sigma_sr, sigma_s=sigma_s)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\zeta = 1 - \beta \left(\frac{\sigma_{sr}}{\sigma_{s}}\right)^2 = "
                r"1 - 0.500 \left(\frac{200.000}{400.000}\right)^2 = 0.875 \ -",
            ),
            ("short", r"\zeta = 0.875 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        beta = 0.5
        sigma_sr = 200.0
        sigma_s = 400.0

        # Object to test
        latex = Form7Dot19DistributionCoefficient(beta=beta, sigma_sr=sigma_sr, sigma_s=sigma_s).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Test Steel material from NEN-EN standards."""

import pytest

from blueprints.materials.steel import DiagramType, SteelMaterial, SteelStrengthClass


class TestSteelMaterial:
    """Test class for the steel material object."""

    def test_default_name(self) -> None:
        """Tests the default name."""
        steel = SteelMaterial()
        assert steel.name == "S 355 (NEN-EN 10025-2)"

    def test_custom_name(self) -> None:
        """Tests the custom name."""
        steel = SteelMaterial(custom_name="Custom Steel")
        assert steel.name == "Custom Steel"

    def test_default_e_modulus(self) -> None:
        """Tests the default modulus of elasticity."""
        steel = SteelMaterial()
        assert steel.e_modulus == 210_000.0

    def test_custom_e_modulus(self) -> None:
        """Tests the custom modulus of elasticity."""
        steel = SteelMaterial(custom_e_modulus=200_000.0)
        assert steel.e_modulus == 200_000.0

    def test_default_poisson_ratio(self) -> None:
        """Tests the default Poisson's ratio."""
        steel = SteelMaterial()
        assert steel.poisson_ratio == 0.3

    def test_custom_poisson_ratio(self) -> None:
        """Tests the custom Poisson's ratio."""
        steel = SteelMaterial(custom_poisson_ratio=0.25)
        assert steel.poisson_ratio == 0.25

    def test_default_thermal_coefficient(self) -> None:
        """Tests the default thermal coefficient."""
        steel = SteelMaterial()
        assert steel.thermal_coefficient == 1.2e-5

    def test_custom_thermal_coefficient(self) -> None:
        """Tests the custom thermal coefficient."""
        steel = SteelMaterial(custom_thermal_coefficient=1.5e-5)
        assert steel.thermal_coefficient == 1.5e-5

    def test_shear_modulus(self) -> None:
        """Tests the shear modulus calculation."""
        steel = SteelMaterial()
        assert steel.shear_modulus == pytest.approx(80_769.23, rel=1e-5)

    def test_default_yield_strength(self) -> None:
        """Tests the default yield strength."""
        steel = SteelMaterial(steel_class=SteelStrengthClass.S275)
        assert steel.yield_strength(10) == 275.0

    def test_custom_yield_strength(self) -> None:
        """Tests the custom yield strength."""
        steel = SteelMaterial(custom_yield_strength=300.0)
        assert steel.yield_strength(10) == 300.0

    def test_default_ultimate_strength(self) -> None:
        """Tests the default ultimate strength."""
        steel = SteelMaterial(steel_class=SteelStrengthClass.S275)
        assert steel.ultimate_strength(thickness=30) == 430.0

    def test_custom_ultimate_strength(self) -> None:
        """Tests the custom ultimate strength."""
        steel = SteelMaterial(custom_ultimate_strength=500.0)
        assert steel.ultimate_strength(thickness=30) == 500.0

    def test_yield_strength_above_40mm(self) -> None:
        """Tests the yield strength for thickness > 40 mm."""
        steel = SteelMaterial(steel_class=SteelStrengthClass.S275)
        assert steel.yield_strength(thickness=50) == 255.0

    def test_ultimate_strength_above_40mm(self) -> None:
        """Tests the ultimate strength for thickness > 40 mm."""
        steel = SteelMaterial(steel_class=SteelStrengthClass.S275)
        assert steel.ultimate_strength(thickness=50.0) == 410.0

    def test_yield_strength_invalid_thickness(self) -> None:
        """Tests yield strength raises ValueError for invalid thickness."""
        steel = SteelMaterial(steel_class=SteelStrengthClass.S275)
        with pytest.raises(ValueError):
            steel.yield_strength(thickness=85.0)

    def test_ultimate_strength_invalid_thickness(self) -> None:
        """Tests ultimate strength raises ValueError for invalid thickness."""
        steel = SteelMaterial(steel_class=SteelStrengthClass.S275)
        with pytest.raises(ValueError):
            steel.ultimate_strength(thickness=85.0)

    def test_diagram_type_default(self) -> None:
        """Tests the default diagram type."""
        steel = SteelMaterial()
        assert steel.diagram_type == DiagramType.BI_LINEAR

    def test_diagram_type_custom(self) -> None:
        """Tests the custom diagram type."""
        steel = SteelMaterial(diagram_type=DiagramType.PARABOLIC)
        assert steel.diagram_type == DiagramType.PARABOLIC

    def test_density_default(self) -> None:
        """Tests the default density."""
        steel = SteelMaterial()
        assert steel.density == 7850.0

    def test_density_custom(self) -> None:
        """Tests the custom density."""
        steel = SteelMaterial(density=8000.0)
        assert steel.density == 8000.0

    def test_eq_with_different_name(self) -> None:
        """Test equality with different name."""
        steel1 = SteelMaterial(steel_class=SteelStrengthClass.S355)
        steel2 = SteelMaterial(steel_class=SteelStrengthClass.S355, custom_name="Custom Name")
        assert steel1 == steel2

    def test_inequality_with_different_e_modulus(self) -> None:
        """Test inequality with different modulus of elasticity."""
        steel1 = SteelMaterial(steel_class=SteelStrengthClass.S355)
        steel2 = SteelMaterial(steel_class=SteelStrengthClass.S355, custom_e_modulus=200_000.0)
        assert steel1 != steel2

    def test_inequality_with_different_steel_class(self) -> None:
        """Test inequality with different steel class."""
        steel1 = SteelMaterial(steel_class=SteelStrengthClass.S355)
        steel2 = SteelMaterial(steel_class=SteelStrengthClass.S275)
        assert steel1 != steel2

    def test_invalid_steel_class(self) -> None:
        """Test invalid steel class raises ValueError."""
        with pytest.raises(ValueError):
            SteelMaterial(steel_class="INVALID_CLASS").yield_strength(thickness=50.0)  # type: ignore[arg-type]

    def test_invalid_yield_above_40mm_thickness(self) -> None:
        """Test invalid thickness raises ValueError."""
        with pytest.raises(ValueError):
            SteelMaterial(steel_class=SteelStrengthClass.S355_H_10219_1).yield_strength(thickness=50.0)

    def test_invalid_ultimate_above_40mm_thickness(self) -> None:
        """Test invalid thickness raises ValueError."""
        with pytest.raises(ValueError):
            SteelMaterial(steel_class=SteelStrengthClass.S355_NH_NLH_10219_1).ultimate_strength(thickness=50.0)



"""Testing formula 6.23 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_23 import Form6Dot23CheckTorsionalMoment
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot23CheckTorsionalMoment:
    """Validation for formula 6.23 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        t_ed = 500.0
        t_rd = 1000.0

        # Object to test
        formula = Form6Dot23CheckTorsionalMoment(t_ed=t_ed, t_rd=t_rd)

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("t_ed", "t_rd"),
        [
            (-500.0, 1000.0),  # t_ed is negative
            (500.0, 0.0),  # t_rd is zero
            (500.0, -1000.0),  # t_rd is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, t_ed: float, t_rd: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot23CheckTorsionalMoment(t_ed=t_ed, t_rd=t_rd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \left( \frac{T_{Ed}}{T_{Rd}} \leq 1.0 \right) \to "
                r"\left( \frac{500.000}{1000.000} \leq 1.0 \right) \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        t_ed = 500.0
        t_rd = 1000.0

        # Object to test
        latex = Form6Dot23CheckTorsionalMoment(t_ed=t_ed, t_rd=t_rd).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 6.40af from EN 1993-1-1:2005: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM, MM2
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot40afHollowSections(Formula):
    r"""Class representing formula 6.40af for [$a_f$] in hollow sections."""

    label = "6.40af_hollow"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        a: MM2,
        h: MM,
        t: MM,
    ) -> None:
        r"""[$a_f$] Calculation of the reduction factor for hollow sections (dimensionless).

        EN 1993-1-1:2005 art.6.2.9.1(5) - Formula (6.40af)

        Parameters
        ----------
        a : MM2
            [$A$] Total cross-sectional area [$mm^2$].
        h : MM
            [$h$] Height of the section [$mm$].
        t : MM
            [$t$] Thickness of the section [$mm$].
        """
        super().__init__()
        self.a = a
        self.h = h
        self.t = t

    @staticmethod
    def _evaluate(
        a: MM2,
        h: MM,
        t: MM,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a=a, h=h, t=t)
        raise_if_less_or_equal_to_zero(denominator=a)

        return min((a - 2 * h * t) / a, 0.5)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.40af."""
        _equation: str = r"\min \left( \frac{A - 2 \cdot h \cdot t}{A}, 0.5 \right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"A": f"{self.a:.{n}f}",
                r" h": f" {self.h:.{n}f}",
                r" t": f" {self.t:.{n}f}",
            },
            False,
        )
        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"A": rf"{self.a:.{n}f} \ mm^2",
                r" h": rf" {self.h:.{n}f} \ mm",
                r" t": rf" {self.t:.{n}f} \ mm",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"a_f",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label=r"=",
            unit="-",
        )


class Form6Dot40afWeldedBoxSections(Formula):
    r"""Class representing formula 6.40af for [$a_f$] in welded box sections."""

    label = "6.40af_welded_box"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        a: MM2,
        h: MM,
        t_w: MM,
    ) -> None:
        r"""[$a_f$] Calculation of the reduction factor for welded box sections (dimensionless).

        EN 1993-1-1:2005 art.6.2.9.1(5) - Formula (6.40af)

        Parameters
        ----------
        a : MM2
            [$A$] Total cross-sectional area [$mm^2$].
        h : MM
            [$h$] Height of the section [$mm$].
        t_w : MM
            [$t_w$] Web thickness [$mm$].
        """
        super().__init__()
        self.a = a
        self.h = h
        self.t_w = t_w

    @staticmethod
    def _evaluate(
        a: MM2,
        h: MM,
        t_w: MM,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a=a, h=h, t_w=t_w)
        raise_if_less_or_equal_to_zero(denominator=a)

        return min((a - 2 * h * t_w) / a, 0.5)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.40af."""
        _equation: str = r"\min \left( \frac{A - 2 \cdot h \cdot t_w}{A}, 0.5 \right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"A": f"{self.a:.{n}f}",
                r" h": f" {self.h:.{n}f}",
                r"t_w": f"{self.t_w:.{n}f}",
            },
            False,
        )
        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"A": rf"{self.a:.{n}f} \ mm^2",
                r" h": rf" {self.h:.{n}f} \ mm",
                r"t_w": rf"{self.t_w:.{n}f} \ mm",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"a_f",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label=r"=",
            unit="-",
        )



"""Fixtures for steel elements."""

from unittest.mock import Mock

import pytest
from shapely.geometry import Point

from blueprints.materials.steel import SteelMaterial
from blueprints.structural_sections._cross_section import CrossSection
from blueprints.structural_sections.steel.steel_element import SteelElement


@pytest.fixture
def mocker() -> Mock:
    """Provide a mocker instance for mocking objects."""
    return Mock()


@pytest.fixture
def mock_cross_section(mocker: Mock) -> CrossSection:
    """Mock a CrossSection object."""
    cross_section: Mock = mocker.Mock(spec=CrossSection)
    cross_section.name = "MockSection"
    cross_section.area = 683  # mm²
    cross_section.perimeter = 400  # mm
    cross_section.centroid = Point(50, 50)
    cross_section.moment_of_inertia_about_y = 2000  # mm⁴
    cross_section.moment_of_inertia_about_z = 3000  # mm⁴
    cross_section.elastic_section_modulus_about_y_positive = 100  # mm³
    cross_section.elastic_section_modulus_about_y_negative = 90  # mm³
    cross_section.elastic_section_modulus_about_z_positive = 80  # mm³
    cross_section.elastic_section_modulus_about_z_negative = 70  # mm³
    cross_section.plastic_section_modulus_about_y = 60  # mm³
    cross_section.plastic_section_modulus_about_z = 50  # mm³
    cross_section.geometry = {"type": "rectangle", "width": 100, "height": 50}
    return cross_section


@pytest.fixture
def mock_material(mocker: Mock) -> SteelMaterial:
    """Mock a SteelMaterial object."""
    material: Mock = mocker.Mock(spec=SteelMaterial)
    material.density = 7850  # kg/m³
    material.yield_strength.return_value = 250  # MPa
    material.ultimate_strength.return_value = 400  # MPa
    return material


@pytest.fixture
def steel_element(mock_cross_section: Mock, mock_material: Mock) -> SteelElement:
    """Create a SteelElement instance using mocked cross-section and material."""
    return SteelElement(cross_section=mock_cross_section, material=mock_material, nominal_thickness=10)



"""Formula 3.2 from EN 1992-1-1:2004: Chapter 3 - Materials."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DAYS, DIMENSIONLESS


class Form3Dot2CoefficientDependentOfConcreteAge(Formula):
    r"""Class representing formula 3.2 for the coefficient [$\beta_{cc}(t)$] which is dependent of the age of concrete."""

    label = "3.2"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        s: DIMENSIONLESS,
        t: DAYS,
    ) -> None:
        r"""[$\beta_{cc}(t)$] Coefficient which is dependent of the age of concrete in days [$-$].

        EN 1992-1-1:2004 art.3.1.2(6) - Formula (3.2)

        Parameters
        ----------
        s : DIMENSIONLESS
            [$s$] Coefficient dependent on the kind of cement [$-$].
            = 0.20 for cement of strength classes CEM 42.5 R, CEM 52.5 N, and CEM 52.5 R (class R);
            = 0.25 for cement of strength classes CEM 32.5 R, CEM 42.5 N (class N);
            = 0.38 for cement of strength class CEM 32.5 N (class S).
            Use your own implementation of this formula or use the SubForm3Dot2CoefficientTypeOfCementS class.
        t : DAYS
            [$t$] Age of concrete in days [$\text{days}$].

        Returns
        -------
        None
        """
        super().__init__()
        self.s = s
        self.t = t

    @staticmethod
    def _evaluate(
        s: DIMENSIONLESS,
        t: DAYS,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        if s < 0:
            raise ValueError(f"Invalid s: {s}. s cannot be negative")
        if t <= 0:
            raise ValueError(f"Invalid t: {t}. t cannot be negative or zero")
        return np.exp(s * (1 - (28 / t) ** (1 / 2)))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 3.2."""
        return LatexFormula(
            return_symbol=r"\beta_{cc}(t)",
            result=f"{self:.{n}f}",
            equation=r"\exp \left( s \cdot \left( 1 - \left( \frac{28}{t} \right) ^{1/2} \right) \right)",
            numeric_equation=rf"\exp \left( {self.s:.{n}f} \cdot \left( 1 - \left( \frac{{28}}{{{self.t:.{n}f}}} \right) ^{{1/2}} \right) \right)",
            comparison_operator_label="=",
        )


class SubForm3Dot2CoefficientTypeOfCementS(Formula):
    r"""Class representing sub-formula for formula 3.2, which calculates the coefficient [$s$] which is dependent on the cement class."""

    source_document = EN_1992_1_1_2004
    label = "3.2"

    def __init__(
        self,
        cement_class: str,
    ) -> None:
        r"""[$s$] Coefficient that depends on the type of cement [$-$].

        EN 1992-1-1:2004 art.3.1.2(6) - s

        Parameters
        ----------
        cement_class : str
            [$cement\_class$] Class of the cement.
                = 'R' for cement of strength classes CEM 42.5 R, CEM 52.5 N, and CEM 52.5 R (class R);
                = 'N' for cement of strength classes CEM 32.5 R, CEM 42.5 N (class N);
                = 'S' for cement of strength class CEM 32.5 N (class S).

        """
        super().__init__()
        self.cement_class = cement_class

    @staticmethod
    def _evaluate(
        cement_class: str,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        match cement_class.lower():
            case "r":
                return 0.20
            case "n":
                return 0.25
            case "s":
                return 0.38
            case _:
                raise ValueError(f"Invalid cement class: {cement_class}. Options: 'R', 'N' or 'S'")

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 3.2s."""
        return LatexFormula(
            return_symbol=r"s",
            result=f"{self:.{n}f}",
            equation=r"\text{cement class}",
            numeric_equation=rf"{self.cement_class}",
            comparison_operator_label=r"\rightarrow",
        )



"""Formula 12.5 and 12.6 from EN 1992-1-1:2004: Chapter 12 - Plain and Lightly Reinforced Concrete Structures."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_12_plain_and_lightly_reinforced_concrete_structures.formula_12_3 import (
    Form12Dot3PlainConcreteShearStress,
)
from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_12_plain_and_lightly_reinforced_concrete_structures.formula_12_4 import (
    Form12Dot4PlainConcreteShearStressComparison,
)
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MPA
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form12Dot5And6PlainConcreteBendingResistance(Formula):
    r"""Class representing formula 12.5 and 12.6 for the calculation of the design bending resistance of plain concrete,
    :math:`f_{cvd}`.

    EN 1992-1-1:2004 art.12.6.1 - Formula (12.5)
    """

    label = "12.5/12.6"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        f_ctd_pl: MPA,
        sigma_cp: MPA | Form12Dot3PlainConcreteShearStress,
        sigma_c_lim: MPA,
    ) -> None:
        r"""[:math:`f_{cvd}`] Design bending resistance of plain concrete [:math=`MPa`].

        EN 1992-1-1:2004 art.12.6.1 - Formula (12.5)

        Parameters
        ----------
        f_ctd_pl : MPA
            [:math=`f_{ctd,pl}`] Design tensile strength of plain concrete [:math=`MPa`].
        sigma_cp : MPA | Form12Dot3PlainConcreteShearStress
            [:math=`\sigma_{cp}`] Compressive stress [:math=`MPa`] or an instance of Form12Dot3PlainConcreteShearStress.
        sigma_c_lim : MPA
            [:math=`\sigma_{c,lim}`] Limiting compressive stress [:math=`MPa`].
        """
        super().__init__()
        self.f_ctd_pl = f_ctd_pl
        self.sigma_cp = sigma_cp
        self.sigma_c_lim = sigma_c_lim

    @staticmethod
    def _evaluate(f_ctd_pl: MPA, sigma_cp: MPA, sigma_c_lim: MPA) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(f_ctd_pl=f_ctd_pl, sigma_cp=sigma_cp)
        comparison = Form12Dot4PlainConcreteShearStressComparison(sigma_cp, sigma_c_lim)
        if comparison:
            return np.sqrt(f_ctd_pl**2 + sigma_cp * f_ctd_pl)
        return np.sqrt(f_ctd_pl**2 + sigma_cp * f_ctd_pl - ((sigma_cp - sigma_c_lim / 2) ** 2))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 12.5. / 12.6."""
        result = self._evaluate(self.f_ctd_pl, self.sigma_cp, self.sigma_c_lim)
        comparison = Form12Dot4PlainConcreteShearStressComparison(self.sigma_cp, self.sigma_c_lim)
        if comparison:
            equation = r"\sqrt{f_{ctd, pl} ^ 2 + \sigma_{cp} \cdot f_{ctd, pl}}"
            numeric_equation = rf"\sqrt{{{self.f_ctd_pl:.{n}f} ^ 2 + {self.sigma_cp:.{n}f} \cdot {self.f_ctd_pl:.{n}f}}}"
        else:
            equation = r"\sqrt{f_{ctd, pl} ^ 2 + \sigma_{cp} \cdot f_{ctd, pl} - \left(\frac{\sigma_{cp} - \sigma_{c, lim}}{2}\right) ^ 2}"
            part1 = f"{self.f_ctd_pl:.{n}f} ^ 2"
            part2 = rf"{self.sigma_cp:.{n}f} \cdot {self.f_ctd_pl:.{n}f}"
            part3 = f"\\left(\\frac{{{self.sigma_cp:.{n}f} - {self.sigma_c_lim:.{n}f}}}{2}\\right) ^ 2"
            numeric_equation = rf"\sqrt{{{part1} + {part2} - {part3}}}"
        return LatexFormula(
            return_symbol=r"f_{cvd}",
            result=f"{result:.{n}f}",
            equation=equation,
            numeric_equation=numeric_equation,
            comparison_operator_label="=",
        )



"""Testing formula 5.3a of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_3a import Form5Dot3aTransverseForceUnbracedMembers
from blueprints.validations import NegativeValueError


class TestForm5Dot3aTransverseForceUnbracedMembers:
    """Validation for formula 5.3a from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        theta_i = 0.003  # -
        n_axial_force = 5  # kN

        # Object to test
        form_5_3a = Form5Dot3aTransverseForceUnbracedMembers(theta_i=theta_i, n_axial_force=n_axial_force)

        # Expected result, manually calculated
        manually_calculated_result = 0.015  # kN

        assert form_5_3a == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("theta_i", "n_axial_force"),
        [
            (-0.003, 5),
            (0.003, -5),
        ],
    )
    def test_raise_error_when_negative_theta_i_is_given(self, theta_i: float, n_axial_force: float) -> None:
        """Test negative values."""
        with pytest.raises(NegativeValueError):
            Form5Dot3aTransverseForceUnbracedMembers(theta_i=theta_i, n_axial_force=n_axial_force)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"H_i = \theta_i \cdot N = 0.003 \cdot 5.000 = 0.015",
            ),
            ("short", r"H_i = 0.015"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        theta_i = 0.003  # -
        n_axial_force = 5  # kN

        # Object to test
        form_5_3a_latex = Form5Dot3aTransverseForceUnbracedMembers(theta_i=theta_i, n_axial_force=n_axial_force).latex()

        actual = {
            "complete": form_5_3a_latex.complete,
            "short": form_5_3a_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 6.76 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_76 import Form6Dot76DesignFatigueStrengthConcrete
from blueprints.validations import LessOrEqualToZeroError


class TestForm6Dot76DesignFatigueStrengthConcrete:
    """Validation for formula 6.76 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        k_1 = 0.85
        beta_cc_t0 = 1.1
        f_cd = 3
        f_ck = 2

        # Object to test
        form_6_76 = Form6Dot76DesignFatigueStrengthConcrete(k_1=k_1, beta_cc_t0=beta_cc_t0, f_cd=f_cd, f_ck=f_ck)

        # Expected result, manually calculated
        manually_calculated_result = 2.78256  # KN

        assert form_6_76 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("k_1", "beta_cc_t0", "f_cd", "f_ck"),
        [
            (-0.85, 1.1, 3, 2),
            (0.85, -1.1, 3, 2),
            (0.85, 1.1, -3, 2),
            (0.85, 1.1, 3, -2),
        ],
    )
    def test_raise_error_when_negative_theta_i_is_given(
        self,
        k_1: float,
        beta_cc_t0: float,
        f_cd: float,
        f_ck: float,
    ) -> None:
        """Test negative values for all arguments."""
        with pytest.raises(LessOrEqualToZeroError):
            Form6Dot76DesignFatigueStrengthConcrete(k_1=k_1, beta_cc_t0=beta_cc_t0, f_cd=f_cd, f_ck=f_ck)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"f_{cd,fat} = k_{1} \cdot β_{cc}(t_0) \cdot f_{cd} \cdot \left(1-\frac{f_{ck}}{250}\right) = "
                r"0.850 \cdot 1.100 \cdot 3.000 \cdot \left(1-\frac{2.000}{250}\right) = 2.783",
            ),
            ("short", r"f_{cd,fat} = 2.783"),
            (
                "string",
                r"f_{cd,fat} = k_{1} \cdot β_{cc}(t_0) \cdot f_{cd} \cdot \left(1-\frac{f_{ck}}{250}\right) = "
                r"0.850 \cdot 1.100 \cdot 3.000 \cdot \left(1-\frac{2.000}{250}\right) = 2.783",
            ),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        k_1 = 0.85
        beta_cc_t0 = 1.1
        f_cd = 3
        f_ck = 2

        # Object to test
        form_6_76_latex = Form6Dot76DesignFatigueStrengthConcrete(k_1=k_1, beta_cc_t0=beta_cc_t0, f_cd=f_cd, f_ck=f_ck).latex()

        actual = {"complete": form_6_76_latex.complete, "short": form_6_76_latex.short, "string": str(form_6_76_latex)}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 5.35 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_35 import Form5Dot35EffectiveDepth
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot35EffectiveDepth:
    """Validation for formula 5.35 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        h = 300.0
        i_s = 50.0

        # Object to test
        formula = Form5Dot35EffectiveDepth(h=h, i_s=i_s)

        # Expected result, manually calculated
        manually_calculated_result = 200.0  # mm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("h", "i_s"),
        [
            (-300.0, 50.0),  # h is negative
            (300.0, -50.0),  # i_s is negative
            (0.0, 50.0),  # h is zero
            (300.0, 0.0),  # i_s is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, h: float, i_s: float) -> None:
        """Test invalid values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot35EffectiveDepth(h=h, i_s=i_s)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"d = \frac{h}{2} + i_s = \frac{300.000}{2} + 50.000 = 200.000 \ mm",
            ),
            ("short", r"d = 200.000 \ mm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        h = 300.0
        i_s = 50.0

        # Object to test
        latex = Form5Dot35EffectiveDepth(h=h, i_s=i_s).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 5.35 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form5Dot35EffectiveDepth(Formula):
    r"""Class representing formula 5.35 for the calculation of the effective depth, [$d$]."""

    label = "5.35"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        h: MM,
        i_s: MM,
    ) -> None:
        r"""[$d$] Effective depth [$mm$].

        EN 1992-1-1:2004 art.5.8.8.3 - Formula (5.35)

        Parameters
        ----------
        h : MM
            [$h$] Total height of the section [$mm$].
        i_s : MM
            [$i_s$] Radius of gyration of the total reinforcement area [$mm$].
        """
        super().__init__()
        self.h = h
        self.i_s = i_s

    @staticmethod
    def _evaluate(
        h: MM,
        i_s: MM,
    ) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(h=h, i_s=i_s)

        return h / 2 + i_s

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.35."""
        return LatexFormula(
            return_symbol=r"d",
            result=f"{self:.{n}f}",
            equation=r"\frac{h}{2} + i_s",
            numeric_equation=rf"\frac{{{self.h:.{n}f}}}{{2}} + {self.i_s:.{n}f}",
            comparison_operator_label="=",
            unit="mm",
        )



"""Testing formula 8.8N of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_8n import (
    Form8Dot8nAnchorageCapacityWeldedTransverseBar,
    SubForm8Dot8nConcreteStress,
    SubForm8Dot8nDesignLengthOfTransverseBar,
)
from blueprints.validations import NegativeValueError


class TestForm8Dot8nAnchorageCapacityWeldedTransverseBar:
    """Validation for formula 8.8N from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        l_td = 100  # mm
        diameter_t = 16  # mm
        sigma_td = 60  # MPa
        f_wd = 400  # kN
        form_8_8n = Form8Dot8nAnchorageCapacityWeldedTransverseBar(
            l_td=l_td,
            diameter_t=diameter_t,
            sigma_td=sigma_td,
            f_wd=f_wd,
        )

        # Expected result, manually calculated
        manually_calculated_result = 96

        assert form_8_8n == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation_maximum(self) -> None:
        """Test the evaluation of the result if the maximum is reached."""
        # Example values
        l_td = 100  # mm
        diameter_t = 16  # mm
        sigma_td = 60  # MPa
        f_wd = 50  # kN
        form_8_8n = Form8Dot8nAnchorageCapacityWeldedTransverseBar(
            l_td=l_td,
            diameter_t=diameter_t,
            sigma_td=sigma_td,
            f_wd=f_wd,
        )

        # Expected result, manually calculated
        manually_calculated_result = 50

        assert form_8_8n == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_l_td_is_given(self) -> None:
        """Test if NegativeValueError is raised when l_td is negative."""
        # Example values
        l_td = -100  # mm
        diameter_t = 16  # mm
        sigma_td = 60  # MPa
        f_wd = 50  # kN

        with pytest.raises(NegativeValueError):
            Form8Dot8nAnchorageCapacityWeldedTransverseBar(
                l_td=l_td,
                diameter_t=diameter_t,
                sigma_td=sigma_td,
                f_wd=f_wd,
            )

    def test_raise_error_when_negative_diameter_t_is_given(self) -> None:
        """Test if NegativeValueError is raised when diameter_t is negative."""
        # Example values
        l_td = 100  # mm
        diameter_t = -16  # mm
        sigma_td = 60  # MPa
        f_wd = 50  # kN

        with pytest.raises(NegativeValueError):
            Form8Dot8nAnchorageCapacityWeldedTransverseBar(
                l_td=l_td,
                diameter_t=diameter_t,
                sigma_td=sigma_td,
                f_wd=f_wd,
            )

    def test_raise_error_when_negative_sigma_td_is_given(self) -> None:
        """Test if NegativeValueError is raised when sigma_td is negative."""
        # Example values
        l_td = 100  # mm
        diameter_t = 16  # mm
        sigma_td = -60  # MPa
        f_wd = 50  # kN

        with pytest.raises(NegativeValueError):
            Form8Dot8nAnchorageCapacityWeldedTransverseBar(
                l_td=l_td,
                diameter_t=diameter_t,
                sigma_td=sigma_td,
                f_wd=f_wd,
            )

    def test_raise_error_when_negative_f_wd_is_given(self) -> None:
        """Test if NegativeValueError is raised when f_wd is negative."""
        # Example values
        l_td = 100  # mm
        diameter_t = 16  # mm
        sigma_td = 60  # MPa
        f_wd = -50  # kN

        with pytest.raises(NegativeValueError):
            Form8Dot8nAnchorageCapacityWeldedTransverseBar(
                l_td=l_td,
                diameter_t=diameter_t,
                sigma_td=sigma_td,
                f_wd=f_wd,
            )

    def test_integration_with_sub_formula_8_8_design_length_of_transverse_bar(
        self,
    ) -> None:
        """Test the integration with sub-formula 8.8 for calculating design length of transverse bar ltd."""
        # Example values
        diameter_t = 16  # mm
        sigma_td = 60  # MPa
        f_wd = 150  # kN
        f_yd = 500  # MPa
        l_t = 100  # mm
        l_td = SubForm8Dot8nDesignLengthOfTransverseBar(
            diameter_t=diameter_t,
            f_yd=f_yd,
            sigma_td=sigma_td,
            l_t=l_t,
        )

        # Object to test
        form_8_8n = Form8Dot8nAnchorageCapacityWeldedTransverseBar(
            l_td=l_td,
            diameter_t=diameter_t,
            sigma_td=sigma_td,
            f_wd=f_wd,
        )

        # Expected result, manually calculated
        manually_calculated_result = 51.434980

        assert form_8_8n == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_integration_with_sub_formula_8_8_concrete_stress(self) -> None:
        """Test the integration with sub-formula 8.8 for calculating concrete stress σtd."""
        # Example values
        diameter_t = 16  # mm
        f_wd = 150  # kN
        l_td = 80  # mm
        f_ctd = 3  # MPa
        sigma_cm = 15  # MPa
        y_function = 0.5  # -
        f_cd = 25  # MPa
        sigma_td = SubForm8Dot8nConcreteStress(
            f_ctd=f_ctd,
            sigma_cm=sigma_cm,
            y_function=y_function,
            f_cd=f_cd,
        )

        # Object to test
        form_8_8n = Form8Dot8nAnchorageCapacityWeldedTransverseBar(
            l_td=l_td,
            diameter_t=diameter_t,
            sigma_td=sigma_td,
            f_wd=f_wd,
        )

        # Expected result, manually calculated
        manually_calculated_result = 46.080000

        assert form_8_8n == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"F_{btd} = \min\left( l_{td} \cdot Ø_t \cdot \sigma_{td}, F_{wd} \right)"
                r" = \min\left( 80.00 \cdot 16.00 \cdot 36.00 / 1000, 150.00 \right) = 46.08",
            ),
            ("short", r"F_{btd} = 46.08"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        diameter_t = 16  # mm
        f_wd = 150  # kN
        l_td = 80  # mm
        f_ctd = 3  # MPa
        sigma_cm = 15  # MPa
        y_function = 0.5  # -
        f_cd = 25  # MPa
        sigma_td = SubForm8Dot8nConcreteStress(
            f_ctd=f_ctd,
            sigma_cm=sigma_cm,
            y_function=y_function,
            f_cd=f_cd,
        )

        # Object to test
        form_8_8n_latex = Form8Dot8nAnchorageCapacityWeldedTransverseBar(
            l_td=l_td,
            diameter_t=diameter_t,
            sigma_td=sigma_td,
            f_wd=f_wd,
        ).latex()

        actual = {"complete": form_8_8n_latex.complete, "short": form_8_8n_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Contains formula 2.22 from CUR 228."""

from blueprints.codes.cur.cur_228 import CUR_228, R_0
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KN_M3, KPA, M
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form2Dot22ModulusHorizontalSubgrade(Formula):
    """Representation of equation 2.22 CUR 228."""

    source_document = CUR_228
    label = "2.22"
    n_decimals: int = 2

    def __init__(self, r: M, e_p: KPA, alpha: DIMENSIONLESS) -> None:
        """Calculates the modulus of horizontal subgrade reaction ($k_h$) using Menard stiffness for r < 0.3 m.

        Parameters
        ----------
        r: M
            The radius of a foundation pile [m]:
            r < 0.3 m
        e_p: KPA
            Elastic modulus of Ménard [kPa]:
            e_p ≈ beta * q_c
                beta: DIMENSIONLESS
                    Dependent on soil type [-]: (table 2.1)
                q_c: KPA
                    Cone resistance [kPa]
        alpha: DIMENSIONLESS
            Factor dependent on soil type [-]: (table 2.1)
        """
        super().__init__()
        self.r = r
        self.e_p = e_p
        self.alpha = alpha

    @staticmethod
    def _evaluate(r: M, e_p: KPA, alpha: DIMENSIONLESS) -> KN_M3:
        """Return the Menard stiffness k_h when r < 0.3 m [kN/m³]."""
        raise_if_negative(e_p=e_p)
        raise_if_less_or_equal_to_zero(r=r, alpha=alpha)
        if r < R_0:
            return e_p / 2 / r / ((4 * 2.65**alpha + 3 * alpha) / 18)
        msg = "Radius is equal to- or larger than 0.3m, use: Eq2Dot21ModulusHorizontalSubgrade"
        raise ValueError(msg)

    def latex(self, n: int = 3) -> LatexFormula:
        """Latex representation of the full equation including result.

        Returns
        -------
        LatexFormula
            Latex representation of the equation

        """
        n = self.n_decimals
        return LatexFormula(
            return_symbol=r"k_{h}",
            result=f"{self:.{n}f}",
            equation=r"\frac{2 \cdot R}{E_{p}} \cdot \frac{4 \cdot 2.65^{\alpha} + 3 \alpha}{18}",
            numeric_equation=rf"\frac{{2 \cdot {self.r:.{n}f}}}{{{self.e_p:.{n}f}}} \cdot \frac{{4 \cdot 2.65^{{{self.alpha:.{n}f}}} + 3 \cdot "
            rf"{self.alpha:.{n}f}}}{{18}}",
            unit="kN/m^3",
        )



"""Testing formula 5.20 from EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_20 import Form5Dot20DesignModulusElasticity
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot20DesignModulusElasticity:
    """Validation for formula 5.20 from EN 1992-1-1:2004."""

    @pytest.fixture
    def form_5_20(self) -> Form5Dot20DesignModulusElasticity:
        """Setup and teardown for test."""
        return Form5Dot20DesignModulusElasticity(e_cm=30000, gamma_ce=1.2)

    def test_evaluation(self, form_5_20: Form5Dot20DesignModulusElasticity) -> None:
        """Test the evaluation of the result."""
        # Expected result, manually calculated
        manually_calculated_result = 30000 / 1.2
        assert form_5_20 == pytest.approx(manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("e_cm", "gamma_ce"),
        [
            (30000, 0),
        ],
    )
    def test_raise_error_when_zero_pars_are_given(self, e_cm: float, gamma_ce: float) -> None:
        """Test zero values for e_cm, gamma_ce."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot20DesignModulusElasticity(e_cm=e_cm, gamma_ce=gamma_ce)

    @pytest.mark.parametrize(
        ("e_cm", "gamma_ce", "expected"),
        [
            (30000, 1.2, 25000),
            (35000, 1.2, 29166.67),
        ],
    )
    def test_properties(self, e_cm: float, gamma_ce: float, expected: float) -> None:
        """Test the properties of the formula."""
        form_5_20 = Form5Dot20DesignModulusElasticity(e_cm=e_cm, gamma_ce=gamma_ce)
        assert form_5_20 == pytest.approx(expected, rel=1e-4)

    def test_latex(self, form_5_20: Form5Dot20DesignModulusElasticity) -> None:
        """Test the latex representation of the formula."""
        # Object to test
        form_5_20_latex = form_5_20.latex()
        expected_complete = r"E_{cd} = \frac{E_{cm}}{\gamma_{CE}} = \frac{30000}{1.2} = 25000.000"
        expected_short = r"E_{cd} = 25000.000"

        assert form_5_20_latex.complete == expected_complete, "Complete representation failed."
        assert form_5_20_latex.short == expected_short, "Short representation failed."



"""Testing formula 9.12 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_9_detailling_and_specific_rules.formula_9_12n import (
    Form9Dot12nMinimumLongitudinalReinforcementColumns,
)
from blueprints.validations import NegativeValueError


class TestForm9Dot12nMinimumLongitudinalReinforcementColumns:
    """Validation for formula 9.12N from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        n_ed = 200  # kN
        f_yd = 500  # MPa
        a_c = 5000  # mm²
        form_9_12n = Form9Dot12nMinimumLongitudinalReinforcementColumns(n_ed=n_ed, f_yd=f_yd, a_c=a_c)

        # Expected result, manually calculated
        manually_calculated_result = 40

        assert form_9_12n == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation_upper_limit(self) -> None:
        """Test the evaluation of the result if the upper limit is reached."""
        # Example values
        n_ed = 200  # kN
        f_yd = 500  # MPa
        a_c = 25000  # mm²
        form_9_12n = Form9Dot12nMinimumLongitudinalReinforcementColumns(n_ed=n_ed, f_yd=f_yd, a_c=a_c)

        # Expected result, manually calculated
        manually_calculated_result = 50

        assert form_9_12n == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_n_ed_is_given(self) -> None:
        """Test whether negative value error is raised if n_ed is negative."""
        n_ed = -200  # kN
        f_yd = 500  # MPa
        a_c = 5000  # mm²

        with pytest.raises(NegativeValueError):
            Form9Dot12nMinimumLongitudinalReinforcementColumns(n_ed=n_ed, f_yd=f_yd, a_c=a_c)

    def test_raise_error_when_negative_f_yd_is_given(self) -> None:
        """Test whether negative value error is raised if f_yd is negative."""
        n_ed = 200  # kN
        f_yd = -500  # MPa
        a_c = 5000  # mm²

        with pytest.raises(NegativeValueError):
            Form9Dot12nMinimumLongitudinalReinforcementColumns(n_ed=n_ed, f_yd=f_yd, a_c=a_c)

    def test_raise_error_when_negative_a_c_is_given(self) -> None:
        """Test whether negative value error is raised if a_c is negative."""
        n_ed = 200  # kN
        f_yd = 500  # MPa
        a_c = -5000  # mm²

        with pytest.raises(NegativeValueError):
            Form9Dot12nMinimumLongitudinalReinforcementColumns(n_ed=n_ed, f_yd=f_yd, a_c=a_c)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                (
                    r"A_{s,min} = \max( \frac{0.10 \cdot N_{Ed}}{f_{yd}}, 0.002 \cdot A_c ) = "
                    r"\max( \frac{0.10 \cdot 200.00}{500.00}, 0.002 \cdot 25000.00 ) = 50.00"
                ),
            ),
            ("short", r"A_{s,min} = 50.00"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        n_ed = 200  # kN
        f_yd = 500  # MPa
        a_c = 25000  # mm²

        # Object to test
        form_9_12n_latex = Form9Dot12nMinimumLongitudinalReinforcementColumns(n_ed=n_ed, f_yd=f_yd, a_c=a_c).latex()

        actual = {"complete": form_9_12n_latex.complete, "short": form_9_12n_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 7.3 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_7_serviceability_limit_state.formula_7_3 import Form7Dot3CoefficientKc


class TestForm7Dot3CoefficientKc:
    """Validation for formula 7.3 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # example values
        f_cr = 457  # KN
        a_ct = 100 * 500  # mm²
        f_ct_eff = 2.9  # MPa
        form_7_3 = Form7Dot3CoefficientKc(f_cr=f_cr, a_ct=a_ct, f_ct_eff=f_ct_eff)

        # expected result, is manually calculated
        manually_calculated_result = 2.83655

        assert form_7_3 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_value_under_0_5(self) -> None:
        """Test that the result will not be smaller than 0.5."""
        # example values
        f_cr = 5  # KN -> This is a very small value, and will result in a very small kc value
        a_ct = 100 * 500  # mm²
        f_ct_eff = 2.9  # MPa
        form_7_3 = Form7Dot3CoefficientKc(f_cr=f_cr, a_ct=a_ct, f_ct_eff=f_ct_eff)

        assert form_7_3 == pytest.approx(expected=0.5, rel=1e-4)

    def test_raise_error_negative_a_ct(self) -> None:
        """Test that an error is raised when a_ct is negative."""
        # example values
        f_cr = 457
        a_ct = -1
        f_ct_eff = 2.9
        with pytest.raises(ValueError):
            Form7Dot3CoefficientKc(f_cr=f_cr, a_ct=a_ct, f_ct_eff=f_ct_eff)

    def test_raise_error_negative_f_ct_eff(self) -> None:
        """Test that an error is raised when f_ct_eff is negative."""
        # example values
        f_cr = 457
        a_ct = 100 * 500
        f_ct_eff = -1
        with pytest.raises(ValueError):
            Form7Dot3CoefficientKc(f_cr=f_cr, a_ct=a_ct, f_ct_eff=f_ct_eff)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"k_c = \max\left(0.9 \cdot \frac{F_{cr}}{A_{ct} \cdot f_{ct,eff}}, 0.5\right) = "
                r"\max\left(0.9 \cdot \frac{457.000}{50000.000 \cdot 2.900}, 0.5\right) = 2.837 \ -",
            ),
            ("short", r"k_c = 2.837 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        f_cr = 457
        a_ct = 100 * 500
        f_ct_eff = 2.9

        # Object to test
        form_7_3_latex = Form7Dot3CoefficientKc(f_cr=f_cr, a_ct=a_ct, f_ct_eff=f_ct_eff).latex()

        actual = {"complete": form_7_3_latex.complete, "short": form_7_3_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Formula 5.28 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KN, KNM
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form5Dot28TotalDesignMoment(Formula):
    """Class representing formula 5.28 for the calculation of the total design moment, [$M_{Ed}$]."""

    label = "5.28"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        m_0ed: KNM,
        beta: DIMENSIONLESS,
        n_ed: KN,
        n_b: KN,
    ) -> None:
        r"""[$M_{Ed}$] Total design moment [$kNm$].

        EN 1992-1-1:2004 art.5.8.8.2(2) - Formula (5.28)

        Parameters
        ----------
        m_0ed : KNM
            [$M_{0Ed}$] First order moment; see also 5.8.8.2 (2) [$kNm$].
        beta : float
            [$\beta$] Factor which depends on distribution of 1st and 2nd order moments, see 5.8.7.3 (2)-(3) [-].
        n_ed : KN
            [$N_{Ed}$] Design value of axial load [$kN$].
        n_b : KN
            [$N_{B}$] Buckling load based on nominal stiffness [$kN$].
        """
        super().__init__()
        self.m_0ed = m_0ed
        self.beta = beta
        self.n_ed = n_ed
        self.n_b = n_b

    @staticmethod
    def _evaluate(
        m_0ed: KNM,
        beta: DIMENSIONLESS,
        n_ed: KN,
        n_b: KN,
    ) -> KN:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            m_0ed=m_0ed,
            beta=beta,
            n_b=n_b,
        )
        raise_if_less_or_equal_to_zero(n_ed=n_ed)

        # When n_b / n_ed is equal to 1, the formula will divide by zero.
        # The spirit of the equation is to raise the second order moment to infinity when n_b / n_ed is equal to 1.
        # This is not possible in numeric calculations, so we will use a large number instead.
        return 1e9 if n_b == n_ed else m_0ed * (1 + beta / (n_b / n_ed - 1))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.28."""
        return LatexFormula(
            return_symbol=r"M_{Ed}",
            result=f"{self:.{n}f}",
            equation=r"M_{0Ed} \cdot \left(1 + \frac{\beta}{\frac{N_{B}}{N_{Ed}} - 1}\right)",
            numeric_equation=rf"{self.m_0ed:.{n}f} \cdot \left(1 + "
            rf"\frac{{{self.beta:.{n}f}}}{{\frac{{{self.n_b:.{n}f}}}{{{self.n_ed:.{n}f}}} - 1}}\right)",
            comparison_operator_label="=",
            unit="kNm",
        )



"""Testing formula 3.8 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_3_materials.formula_3_8 import Form3Dot8TotalShrinkage


class TestForm3Dot8TotalShrinkage:
    """Validation for formula 3.8 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        epsilon_cd = 0.25  # -
        epsilon_ca = 0.33  # -
        form_3_8 = Form3Dot8TotalShrinkage(epsilon_cd=epsilon_cd, epsilon_ca=epsilon_ca)

        # Expected result, manually calculated
        manually_calculated_result = 0.58

        assert form_3_8 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation_2(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        epsilon_cd = -0.25  # -
        epsilon_ca = 0.33  # -
        form_3_8 = Form3Dot8TotalShrinkage(epsilon_cd=epsilon_cd, epsilon_ca=epsilon_ca)

        # Expected result, manually calculated
        manually_calculated_result = 0.08

        assert form_3_8 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\epsilon_{cs} = \epsilon_{cd} + \epsilon_{ca} = 0.250 + 0.330 = 0.580",
            ),
            ("short", r"\epsilon_{cs} = 0.580"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        epsilon_cd = 0.25  # -
        epsilon_ca = 0.33  # -

        # Object to test
        form_3_8_latex = Form3Dot8TotalShrinkage(epsilon_cd=epsilon_cd, epsilon_ca=epsilon_ca).latex()

        actual = {"complete": form_3_8_latex.complete, "short": form_3_8_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 6.49 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_49 import Form6Dot49AppliedPunchingShearStress
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot49AppliedPunchingShearStress:
    """Validation for formula 6.49 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        v_ed_red = 1000000.0
        u = 200.0
        d = 300.0

        # Object to test
        formula = Form6Dot49AppliedPunchingShearStress(v_ed_red=v_ed_red, u=u, d=d)

        # Expected result, manually calculated
        manually_calculated_result = 16.6667  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("v_ed_red", "u", "d"),
        [
            (-1000000.0, 200.0, 300.0),  # v_ed_red is negative
            (1000000.0, -200.0, 300.0),  # u is negative
            (1000000.0, 200.0, -300.0),  # d is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, v_ed_red: float, u: float, d: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot49AppliedPunchingShearStress(v_ed_red=v_ed_red, u=u, d=d)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"v_{Ed} = \frac{V_{Ed,red}}{u \cdot d} = \frac{1000000.000}{200.000 \cdot 300.000} = 16.667 \ MPa",
            ),
            ("short", r"v_{Ed} = 16.667 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        v_ed_red = 1000000.0
        u = 200.0
        d = 300.0

        # Object to test
        latex = Form6Dot49AppliedPunchingShearStress(v_ed_red=v_ed_red, u=u, d=d).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 6.3 from EN 1993-1-1:2005: Chapter 6 - Ultimate limit state."""

from collections.abc import Sequence

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_lists_differ_in_length, raise_if_negative


class Form6Dot3MinDeductionAreaStaggeredFastenerHoles(Formula):
    """Class representing formula 6.3 for the calculation of the minimum area deduction for staggered fastener holes [$A_{deduction}$]."""

    label = "6.3"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        t: MM,
        n: MM,
        d_0: MM,
        s: Sequence[MM],
        p: Sequence[MM],
    ) -> None:
        """[$A_{deduction}$] Calculation of the area deduction for staggered fastener holes [$mm^2$].

        EN 1993-1-1:2005 art.6.2.2.2 (4) b) - Formula (6.3)
        section (4) a) should be handled separately.

        Parameters
        ----------
        t : MM
            [$t$] Thickness [$mm$].
        n : MM
            [$n$] Number of holes extending in any diagonal or zig-zag line progressively across the member, see Figure 6.1 [$mm$].
        d_0 : MM
            [$d_0$] Diameter of hole [$mm$].
        s : Sequence[MM]
            [$s$] Staggered pitch, the spacing of the centres of two consecutive holes in the
            chain measured parallel to the member axis [$mm$].
        p : Sequence[MM]
            [$p$] Spacing of the centres of the same two holes measured perpendicular to the member axis [$mm$].
        """
        super().__init__()
        self.t = t
        self.n = n
        self.d_0 = d_0
        self.s = s
        self.p = p

    @staticmethod
    def _evaluate(
        t: MM,
        n: MM,
        d_0: MM,
        s: Sequence[MM],
        p: Sequence[MM],
    ) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(t=t, n=n, d_0=d_0)
        raise_if_less_or_equal_to_zero(t=t, n=n, d_0=d_0)
        raise_if_lists_differ_in_length(s=s, p=p)
        for s_i, p_i in zip(s, p):
            raise_if_negative(s=s_i, p=p_i)
            raise_if_less_or_equal_to_zero(s=s_i, p=p_i)

        return t * (n * d_0 - sum((s_i**2) / (4 * p_i) for s_i, p_i in zip(s, p)))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.3."""
        _equation: str = r"t \left( n \cdot d_0 - \sum \frac{s^2}{4 \cdot p} \right)"
        _numeric_equation: str = (
            rf"{self.t:.{n}f} \left( {self.n:.{n}f} \cdot {self.d_0:.{n}f} - \left( \frac{{{self.s[0]:.{n}f}^2}}"
            rf"{{4 \cdot {self.p[0]:.{n}f}}}"
        )
        for s_i, p_i in zip(self.s[1:], self.p[1:]):
            _numeric_equation += rf" + \frac{{{s_i:.{n}f}^2}}{{4 \cdot {p_i:.{n}f}}}"
        _numeric_equation += r" \right) \right)"
        return LatexFormula(
            return_symbol=r"A_{deduction}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="mm^2",
        )



"""Testing formula 6.26 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_26 import Form6Dot26ShearStressInWall
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot26ShearStressInWall:
    """Validation for formula 6.26 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        t_ed = 100.0
        a_k = 200.0

        # Object to test
        formula = Form6Dot26ShearStressInWall(t_ed=t_ed, a_k=a_k)

        # Expected result, manually calculated
        manually_calculated_result = 0.25  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("t_ed", "a_k"),
        [
            (-100.0, 200.0),  # t_ed is negative
            (100.0, -200.0),  # a_k is negative
            (100.0, 0.0),  # a_k is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, t_ed: float, a_k: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot26ShearStressInWall(t_ed=t_ed, a_k=a_k)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\tau_{t,i}t_{ef,i} = \frac{T_{Ed}}{2 \cdot A_{k}} = \frac{100.000}{2 \cdot 200.000} = 0.250 \ N/mm",
            ),
            ("short", r"\tau_{t,i}t_{ef,i} = 0.250 \ N/mm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        t_ed = 100.0
        a_k = 200.0

        # Object to test
        latex = Form6Dot26ShearStressInWall(t_ed=t_ed, a_k=a_k).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 5.11n of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_11n import Form5Dot11nShearSlendernessCorrectionFactor
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot11nShearSlendernessCorrectionFactor:
    """Validation for formula 5.11N from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        lambda_factor = 0.5  # -

        # Object to test
        form_5_11n = Form5Dot11nShearSlendernessCorrectionFactor(lambda_factor=lambda_factor)

        # Expected result, manually calculated
        manually_calculated_result = 0.40824  # -

        assert form_5_11n == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        "lambda_factor",
        [
            0,
            -0.3,
        ],
    )
    def test_raise_error_when_negative_values_or_zero_are_given(
        self,
        lambda_factor: float,
    ) -> None:
        """Test negative or zero values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot11nShearSlendernessCorrectionFactor(lambda_factor=lambda_factor)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            ("complete", r"k_{λ} = \sqrt{\frac{λ}{3}} = \sqrt\frac{0.3}{3} = 0.316"),
            ("short", "k_{λ} = 0.316"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        lambda_factor = 0.3  # -

        # Object to test
        latex = Form5Dot11nShearSlendernessCorrectionFactor(lambda_factor=lambda_factor).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Formula 6.39 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MM, MM2, NMM, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot39BetaCoefficient(Formula):
    r"""Class representing formula 6.39 for the calculation of the beta coefficient, [$\beta$]."""

    label = "6.39"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        k: DIMENSIONLESS,
        m_ed: NMM,
        v_ed: N,
        u_1: MM,
        w_1: MM2,
    ) -> None:
        r"""[$\beta$] Beta coefficient [$-$].

        EN 1992-1-1:2004 art.6.4.3(3) - Formula (6.39)

        Parameters
        ----------
        k : DIMENSIONLESS
            [$k$] Coefficient dependent on the ratio between the column dimensions c1 and c2 [$-$].
        m_ed : NMM
            [$M_{Ed}$] Design value of the applied moment [$Nmm$].
        v_ed : N
            [$V_{Ed}$] Design value of the applied shear force [$N$].
        u_1 : MM
            [$u_1$] Length of the basic control perimeter [$mm$].
        w_1 : MM2
            [$W_1$] Distribution of shear as illustrated in Figure 6.19 [$mm^2$].
        """
        super().__init__()
        self.k = k
        self.m_ed = m_ed
        self.v_ed = v_ed
        self.u_1 = u_1
        self.w_1 = w_1

    @staticmethod
    def _evaluate(
        k: DIMENSIONLESS,
        m_ed: NMM,
        v_ed: N,
        u_1: MM,
        w_1: MM,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            k=k,
            m_ed=m_ed,
            u_1=u_1,
        )
        raise_if_less_or_equal_to_zero(v_ed=v_ed, w_1=w_1)

        return 1 + k * m_ed / v_ed * u_1 / w_1

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.39."""
        return LatexFormula(
            return_symbol=r"\beta",
            result=f"{self:.{n}f}",
            equation=r"1 + k \cdot \frac{M_{Ed}}{V_{Ed}} \cdot \frac{u_1}{W_1}",
            numeric_equation=rf"1 + {self.k:.{n}f} \cdot \frac{{{self.m_ed:.{n}f}}}{{{self.v_ed:.{n}f}}} \cdot \frac{{{self.u_1:.{n}f}}}{{{self.w_1:.{n}f}}}",  # noqa: E501
            comparison_operator_label="=",
            unit="-",
        )



"""Formula 5.27 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MPA
from blueprints.validations import raise_if_negative


class Form5Dot27EffectiveDesignModulusElasticity(Formula):
    """Class representing formula 5.27 for the calculation of the effective design modulus of elasticity, [$E_{cd,eff}$]."""

    label = "5.27"
    source_document = EN_1992_1_1_2004

    def __init__(self, e_cd: MPA, phi_ef: DIMENSIONLESS) -> None:
        r"""[$E_{cd,eff}$] Effective design modulus of elasticity.

        EN 1992-1-1:2004 art.5.8.6(3) - Formula (5.27)

        Parameters
        ----------
        e_cd : MPA
            [$E_{cd}$] is the design modulus of elasticity of concrete according to 5.8.6 (3).
        phi_ef : DIMENSIONLESS
            [$\phi_{ef}$] is the effective creep coefficient; same value as for columns may be used.
        """
        super().__init__()
        self.e_cd = e_cd
        self.phi_ef = phi_ef

    @staticmethod
    def _evaluate(e_cd: MPA, phi_ef: DIMENSIONLESS) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(e_cd=e_cd, phi_ef=phi_ef)
        return e_cd / (1 + phi_ef)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.27."""
        return LatexFormula(
            return_symbol=r"E_{cd,eff}",
            result=f"{self:.{n}f}",
            equation=r"\frac{E_{cd}}{1 + \phi_{ef}}",
            numeric_equation=rf"\frac{{{self.e_cd:.{n}f}}}{{1 + {self.phi_ef:.{n}f}}}",
            comparison_operator_label="=",
        )



"""Testing formula 5.47 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_47 import (
    Form5Dot47UpperCharacteristicPrestressingValue,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot47UpperCharacteristicPrestressingValue:
    """Validation for formula 5.47 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        r_sup = 1.05
        p_m_t = 100.0

        # Object to test
        formula = Form5Dot47UpperCharacteristicPrestressingValue(r_sup=r_sup, p_m_t=p_m_t)

        # Expected result, manually calculated
        manually_calculated_result = 105.0

        assert formula == pytest.approx(manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("r_sup", "p_m_t"),
        [
            (-1.05, 100.0),  # r_sup is negative
            (1.05, -100.0),  # p_m_t is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, r_sup: float, p_m_t: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form5Dot47UpperCharacteristicPrestressingValue(r_sup=r_sup, p_m_t=p_m_t)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"P_{k,sup} = r_{sup} \cdot P_{m,t}(x) = 1.050 \cdot 100.000 = 105.000 \ kN",
            ),
            ("short", r"P_{k,sup} = 105.000 \ kN"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        r_sup = 1.05
        p_m_t = 100.0

        # Object to test
        latex = Form5Dot47UpperCharacteristicPrestressingValue(r_sup=r_sup, p_m_t=p_m_t).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 8.11 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_3 import (
    Form8Dot3RequiredAnchorageLength,
)
from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_11 import (
    Form8Dot11MinimumDesignLapLength,
)
from blueprints.validations import NegativeValueError


class TestForm8Dot11MinimumDesignLapLength:
    """Validation for formula 8.11 from EN 1992-1-1:2004."""

    def test_evaluation_first_term_decisive(self) -> None:
        """Test the evaluation of the result."""
        # example values
        alpha_6 = 1  # [-]
        l_b_rqd = 1200  # mm
        diameter = 16  # mm
        form_8_11 = Form8Dot11MinimumDesignLapLength(
            alpha_6=alpha_6,
            l_b_rqd=l_b_rqd,
            diameter=diameter,
        )

        # manually calculated result
        manually_calculated_result = 360  # mm

        assert form_8_11 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation_second_term_decisive(self) -> None:
        """Test the evaluation of the result."""
        # example values
        alpha_6 = 1  # [-]
        l_b_rqd = 450  # mm
        diameter = 32  # mm
        form_8_11 = Form8Dot11MinimumDesignLapLength(
            alpha_6=alpha_6,
            l_b_rqd=l_b_rqd,
            diameter=diameter,
        )

        # manually calculated result
        manually_calculated_result = 480  # mm

        assert form_8_11 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation_third_term_decisive(self) -> None:
        """Test the evaluation of the result."""
        # example values
        alpha_6 = 1  # [-]
        l_b_rqd = 450  # mm
        diameter = 8  # mm
        form_8_11 = Form8Dot11MinimumDesignLapLength(
            alpha_6=alpha_6,
            l_b_rqd=l_b_rqd,
            diameter=diameter,
        )

        # manually calculated result
        manually_calculated_result = 200  # mm

        assert form_8_11 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_if_alpha_6_is_negative(self) -> None:
        """Test that a NegativeValueError is raised if alpha_6 is negative."""
        # example values
        alpha_6 = -1  # [-]
        l_b_rqd = 450  # mm
        diameter = 8  # mm

        with pytest.raises(NegativeValueError):
            Form8Dot11MinimumDesignLapLength(
                alpha_6=alpha_6,
                l_b_rqd=l_b_rqd,
                diameter=diameter,
            )

    def test_raise_error_if_l_b_rqd_is_negative(self) -> None:
        """Test that a NegativeValueError is raised if l_b_rqd is negative."""
        # example values
        alpha_6 = 1  # [-]
        l_b_rqd = -450  # mm
        diameter = 8  # mm

        with pytest.raises(NegativeValueError):
            Form8Dot11MinimumDesignLapLength(
                alpha_6=alpha_6,
                l_b_rqd=l_b_rqd,
                diameter=diameter,
            )

    def test_raise_error_if_diameter_is_negative(self) -> None:
        """Test that a NegativeValueError is raised if diameter is negative."""
        # example values
        alpha_6 = 1  # [-]
        l_b_rqd = 450  # mm
        diameter = -8  # mm

        with pytest.raises(NegativeValueError):
            Form8Dot11MinimumDesignLapLength(
                alpha_6=alpha_6,
                l_b_rqd=l_b_rqd,
                diameter=diameter,
            )

    def test_integration_with_form_8_3(self) -> None:
        """Test the integration with formula 8.3."""
        # example values
        diameter = 8  # mm
        alpha_6 = 1  # [-]
        sigma_sd = 500  # MPa
        f_bd = 2.5  # MPa
        l_b_rqd = Form8Dot3RequiredAnchorageLength(
            diameter=diameter,
            sigma_sd=sigma_sd,
            f_bd=f_bd,
        )

        form_8_11 = Form8Dot11MinimumDesignLapLength(
            alpha_6=alpha_6,
            l_b_rqd=l_b_rqd,
            diameter=diameter,
        )

        # manually calculated result
        manually_calculated_result = 200  # mm

        assert form_8_11 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            (
                "complete",
                r"l_{Ø,min} = \max \left\{0.3 \cdot \alpha_6 \cdot l_{b,rqd}; 15 \cdot Ø; 200 \ \text{mm}\right\} = \max \left\{0.3 \cdot 1.000 "
                r"\cdot 450.000; 15 \cdot 8; 200\right\} = 200.000",
            ),
            ("short", r"l_{Ø,min} = 200.000"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the LaTeX representation."""
        # example values
        alpha_6 = 1
        l_b_rqd = 450
        diameter = 8

        latex = Form8Dot11MinimumDesignLapLength(
            alpha_6=alpha_6,
            l_b_rqd=l_b_rqd,
            diameter=diameter,
        ).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Testing formula 5.12N of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_12n import (
    Form5Dot12nRatioDistancePointZeroAndMaxMoment,
)
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot12nRatioDistancePointZeroAndMaxMoment:
    """Validation for formula 5.12N from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        m_sd = 100  # kNm
        v_sd = 10  # kN
        d = 0.3  # m

        # Object to test
        formula = Form5Dot12nRatioDistancePointZeroAndMaxMoment(m_sd=m_sd, v_sd=v_sd, d=d)

        # Expected result, manually calculated
        manually_calculated_result = 33.333  # -

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("m_sd", "v_sd", "d"),
        [
            (-100, 10, 0.3),
            (100, -10, 0.3),
            (100, 10, -0.3),
            (0, 10, 0.3),
            (100, 0, 0.3),
            (100, 10, 0),
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, m_sd: float, v_sd: float, d: float) -> None:
        """Test negative values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot12nRatioDistancePointZeroAndMaxMoment(m_sd=m_sd, v_sd=v_sd, d=d)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"λ = \frac{M_{sd}}{V_{sd} \cdot d} = \frac{100.000}{10.000 \cdot 0.300} = 33.333",
            ),
            ("short", r"λ = 33.333"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        m_sd = 100
        v_sd = 10
        d = 0.3

        # Object to test
        latex = Form5Dot12nRatioDistancePointZeroAndMaxMoment(m_sd=m_sd, v_sd=v_sd, d=d).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 6.17 from EN 1992-1-1:2004: Chapter 6 - Ultimate limit state."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM
from blueprints.validations import raise_if_negative


class Form6Dot17NominalWebWidth(Formula):
    r"""Class representing formula 6.17 for the calculation of the nominal web width (for non-grouted ducts, grouted plastic ducts
    and unbonded tendons), [$b_{w,nom}$].
    """

    label = "6.17"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        b_w: MM,
        diameters: list[MM],
    ) -> None:
        r"""[$b_{w,nom}$] Nominal web width [$mm$].

        EN 1992-1-1:2004 art.6.2.3(6) - Formula (6.17)

        Parameters
        ----------
        b_w : MM
            [$b_{w}$] Web width [$mm$].
        diameters : list[MM]
            [$⌀$] Diameters of the reinforcement bars for the most unfavourable level [$mm$].
        """
        super().__init__()
        self.b_w = b_w
        self.diameters = diameters

    @staticmethod
    def _evaluate(
        b_w: MM,
        diameters: list[MM],
    ) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(b_w=b_w)
        for diameter in diameters:
            raise_if_negative(diameter=diameter)

        return b_w - 1.2 * sum(diameters)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.17."""
        return LatexFormula(
            return_symbol=r"b_{w,nom}",
            result=f"{self:.{n}f}",
            equation=r"b_{w} - 1.2 \cdot \sum(⌀)",
            numeric_equation=rf"{self.b_w:.{n}f} - 1.2 \cdot \left({' + '.join([f'{d:.{n}f}' for d in self.diameters])} \right)",
            comparison_operator_label="=",
            unit="mm",
        )



"""Testing formula 6.21 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_21 import Form6Dot21CheckTransverseReinforcement
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot21CheckTransverseReinforcement:
    """Validation for formula 6.21 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        a_sf = 100.0
        f_yd = 500.0
        s_f = 200.0
        v_ed = 2.0
        h_f = 300.0
        theta_f = 45.0

        # Object to test
        formula = Form6Dot21CheckTransverseReinforcement(a_sf=a_sf, f_yd=f_yd, s_f=s_f, v_ed=v_ed, h_f=h_f, theta_f=theta_f)

        # Expected result, manually calculated
        expected_result = False

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("a_sf", "f_yd", "s_f", "v_ed", "h_f", "theta_f"),
        [
            (-100.0, 500.0, 200.0, 2.0, 300.0, 45.0),  # a_sf is negative
            (100.0, -500.0, 200.0, 2.0, 300.0, 45.0),  # f_yd is negative
            (100.0, 500.0, -200.0, 2.0, 300.0, 45.0),  # s_f is negative
            (100.0, 500.0, 200.0, -2.0, 300.0, 45.0),  # v_ed is negative
            (100.0, 500.0, 200.0, 2.0, -300.0, 45.0),  # h_f is negative
            (100.0, 500.0, 200.0, 2.0, 300.0, -45.0),  # theta_f is negative
            (100.0, 500.0, 0.0, 2.0, 300.0, 45.0),  # s_f is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, a_sf: float, f_yd: float, s_f: float, v_ed: float, h_f: float, theta_f: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot21CheckTransverseReinforcement(a_sf=a_sf, f_yd=f_yd, s_f=s_f, v_ed=v_ed, h_f=h_f, theta_f=theta_f)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \left( \frac{A_{sf} \cdot f_{yd}}{s_{f}} \geq \frac{v_{Ed} \cdot h_{f}}{\cot(\theta_{f})} \right) \to"
                r" \left( \frac{100.000 \cdot 500.000}{200.000} \geq \frac{2.000 \cdot 300.000}{\cot(45.000)} \right) \to \text{Not OK}",
            ),
            ("short", r"CHECK \to \text{Not OK}"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        a_sf = 100.0
        f_yd = 500.0
        s_f = 200.0
        v_ed = 2.0
        h_f = 300.0
        theta_f = 45.0

        # Object to test
        latex = Form6Dot21CheckTransverseReinforcement(a_sf=a_sf, f_yd=f_yd, s_f=s_f, v_ed=v_ed, h_f=h_f, theta_f=theta_f).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 6.10 from EN 1993-1-1:2005: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM2, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot10NcRdClass1And2And3(Formula):
    r"""Class representing formula 6.10 for the calculation of [$N_{c,Rd}$]."""

    label = "6.10"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        a: MM2,
        f_y: MPA,
        gamma_m0: DIMENSIONLESS,
    ) -> None:
        r"""[$N_{c,Rd}$] Calculation of the design resistance of the cross-section for uniform compression [$N$].
        This equation is only valid for cross-sections with class 1, 2, or 3.

        EN 1993-1-1:2005 art.6.2.4(2) - Formula (6.10)

        Parameters
        ----------
        a : MM2
            [$A$] Cross-sectional area [$mm^2$].
        f_y : MPA
            [$f_y$] Yield strength of the material [$MPa$].
        gamma_m0 : DIMENSIONLESS
            [$\gamma_{M0}$] Partial safety factor for resistance of cross-sections.
        """
        super().__init__()
        self.a = a
        self.f_y = f_y
        self.gamma_m0 = gamma_m0

    @staticmethod
    def _evaluate(
        a: MM2,
        f_y: MPA,
        gamma_m0: DIMENSIONLESS,
    ) -> N:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a=a, f_y=f_y)
        raise_if_less_or_equal_to_zero(gamma_m0=gamma_m0)

        return (a * f_y) / gamma_m0

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.10."""
        _equation: str = r"\frac{A \cdot f_y}{\gamma_{M0}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"A": f"{self.a:.{n}f}",
                r"f_y": f"{self.f_y:.{n}f}",
                r"\gamma_{M0}": f"{self.gamma_m0:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"N_{c,Rd}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="N",
        )



"""Formula 8.5 from EN 1992-1-1:2004: Chapter 8: Detailing of reinforcement and prestressing tendons."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import RATIO
from blueprints.validations import raise_if_negative


class Form8Dot5ProductAlphas235(Formula):
    r"""Class representing formula 8.5 for the calculating the product of [$\alpha_{2}$], [$\alpha_{3}$] [$\alpha_{5}$] [$-$]."""

    label = "8.5"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        alpha_2: RATIO,
        alpha_3: RATIO,
        alpha_5: RATIO,
    ) -> None:
        r"""Calculate the product of [$\alpha_{2}$], [$\alpha_{3}$] and [$\alpha_{5}$] [$-$].

        EN 1992-1-1:2004 art.8.4.4(1) - Formula (8.5) prescribes that [$(\alpha_{2} \cdot \alpha_{3} \cdot \alpha_{5}) \ge 0.7$].
        Used by EN 1992-1-1:2004 art.8.4.4(1) - Formula (8.4)

        Parameters
        ----------
        alpha_2 : RATIO
            [$\alpha_{2}$] Coefficient for the effect of minimum concrete cover (see figure 8.3) [$-$].

            [$= 1.0$] for bars in compression.

            [$= 1 - 0.15 \cdot (c_{d} - \varnothing) / \varnothing \le 1$] with a minimum of [$0.7$] for straight bars in tension.

            [$= 1 - 0.15 \cdot (c_{d} - 3 \cdot \varnothing) / \varnothing \le 1$] with a minimum of [$0.7$] for bars other than
            straight in tension (see figure 8.1 (b), (c) and (d)).

            Note: see figure 8.3 for values of [$c_{d}$].
        alpha_3 : RATIO
            [$\alpha_{3}$] Coefficient for the effect of confinement by transverse reinforcement [$-$].

            [$= 1.0$] for bars in compression.

            [$= 1 - K \cdot \lambda \le 1$] with a minimum of [$0.7$] for bars in tension.

            Where: [$\lambda = (\Sigma A_{st} - \Sigma A_{st,min}) / A_{s}$].

            Where: [$\Sigma A_{st,min}$] = cross-sectional area of the minimum transverse
            reinforcement [$= 0.25 \cdot A_{s}$] for beams and [$0$] for slabs.

            Note: see figure 8.4 for values of [$K, A_{s}$] and [$A_{st}$].

        alpha_5 : RATIO
            [$\alpha_{5}$] Coefficient for the effect of the pressure transverse to the plane of splitting
            along the design anchorage length [$l_{bd}$] (see 8.6) [$-$].

            [$= 1 - 0.04 \cdot p \le 1$] with a minimum of [$0.7$] for all types of anchorage in compression.

            Where: [$p$] = transverse pressure at ultimate limit state along [$l_{bd}$] [$MPa$].
        """
        super().__init__()
        self.alpha_2 = alpha_2
        self.alpha_3 = alpha_3
        self.alpha_5 = alpha_5

    @staticmethod
    def _evaluate(
        alpha_2: RATIO,
        alpha_3: RATIO,
        alpha_5: RATIO,
    ) -> RATIO:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            alpha_2=alpha_2,
            alpha_3=alpha_3,
            alpha_5=alpha_5,
        )

        # The product of alpha values 2, 3, 5 can't be smaller than 0.7
        return max(alpha_2 * alpha_3 * alpha_5, 0.7)

    def latex(self, n: int = 2) -> LatexFormula:
        """Returns a LatexFormula representation of the formula."""
        return LatexFormula(
            return_symbol="",
            result=f"{self:.{n}f}",
            equation=r"\alpha_2 \cdot \alpha_3 \cdot \alpha_5 \ge 0.7",
            numeric_equation=rf"{self.alpha_2} \cdot {self.alpha_3} \cdot {self.alpha_5} \ge 0.7",
            comparison_operator_label="\\to",
        )



"""Formula 5.8 from EN 1993-1-1:2005: Chapter 5 - Structural Analysis."""

import numpy as np

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import ComparisonFormula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM2, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form5Dot8CheckSlenderness(ComparisonFormula):
    r"""Class representing formula 5.8 for check of slenderness."""

    label = "5.8"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        lambda_bar: DIMENSIONLESS,
        a: MM2,
        f_y: MPA,
        n_ed: N,
    ) -> None:
        r"""Check the slenderness ratio.

        EN 1993-1-1:2005 art.5.3.2(6) - Formula (5.8)

        Parameters
        ----------
        lambda_bar : DIMENSIONLESS
            [$\overline{\lambda}$] In-plane non-dimensional slenderness calculated for the member
            considered as hinged at its ends [-].
        a : MM2
            [$A$] Cross-sectional area [$mm^2$].
        f_y : MPA
            [$f_y$] Yield strength [$MPa$].
        n_ed : N
            [$N_{Ed}$] Design value of the compression force [$N$].
        """
        super().__init__()
        self.lambda_bar = lambda_bar
        self.a = a
        self.f_y = f_y
        self.n_ed = n_ed

    @staticmethod
    def _evaluate_lhs(lambda_bar: DIMENSIONLESS, *_args, **_kwargs) -> float:
        """Evaluates the left-hand side of the comparison. See __init__ for details."""
        raise_if_negative(lambda_bar=lambda_bar)
        return lambda_bar

    @staticmethod
    def _evaluate_rhs(a: MM2, f_y: MPA, n_ed: N, *_args, **_kwargs) -> float:
        """Evaluates the right-hand side of the comparison. See __init__ for details."""
        raise_if_less_or_equal_to_zero(n_ed=n_ed)
        raise_if_negative(a=a, f_y=f_y)
        return 0.5 * np.sqrt(a * f_y / n_ed)

    @property
    def unity_check(self) -> float:
        """Returns the unity check value."""
        return self.lhs / self.rhs

    @staticmethod
    def _evaluate(
        lambda_bar: DIMENSIONLESS,
        a: MM2,
        f_y: MPA,
        n_ed: N,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        lhs = Form5Dot8CheckSlenderness._evaluate_lhs(lambda_bar=lambda_bar)
        rhs = Form5Dot8CheckSlenderness._evaluate_rhs(a=a, f_y=f_y, n_ed=n_ed)
        return lhs > rhs

    def __bool__(self) -> bool:
        """Allow truth-checking of the check object itself."""
        return self._evaluate(
            lambda_bar=self.lambda_bar,
            a=self.a,
            f_y=self.f_y,
            n_ed=self.n_ed,
        )

    def latex(self, n: int = 2) -> LatexFormula:
        """Returns LatexFormula object for formula 5.8."""
        _equation: str = r"\left( \overline{\lambda} > 0.5 \sqrt{\frac{A \cdot f_{y}}{N_{Ed}}} \right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"\lambda": f"{self.lambda_bar:.{n}f}",
                "A": f"{self.a:.{n}f}",
                "f_{y}": f"{self.f_y:.{n}f}",
                "N_{Ed}": f"{self.n_ed:.{n}f}",
            },
            unique_symbol_check=False,
        )
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="\\to",
            unit="",
        )



"""Formula 7.13 from EN 1992-1-1:2004: Chapter 7 - Serviceability limit state (SLS)."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form7Dot13CoefficientK2(Formula):
    r"""Class representing formula 7.13 for the calculation of [$k_2$]."""

    label = "7.13"
    source_document = EN_1992_1_1_2004

    def __init__(self, epsilon_1: DIMENSIONLESS, epsilon_2: DIMENSIONLESS) -> None:
        r"""[$k_2$] Calculation of the coefficient for distribution of strain [$-$].

        EN 1992-1-1:2004 art.7.3.4(3) - Formula (7.13)

        Parameters
        ----------
        epsilon_1 : DIMENSIONLESS
            [$\epsilon_1$] Greater tensile strain at the boundaries of the section considered [$-$].
        epsilon_2 : DIMENSIONLESS
            [$\epsilon_2$] Lesser tensile strain at the boundaries of the section considered [$-$].
        """
        super().__init__()
        self.epsilon_1 = epsilon_1
        self.epsilon_2 = epsilon_2

    @staticmethod
    def _evaluate(epsilon_1: DIMENSIONLESS, epsilon_2: DIMENSIONLESS) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(epsilon_1=epsilon_1)
        raise_if_negative(epsilon_2=epsilon_2)

        return (epsilon_1 + epsilon_2) / (2 * epsilon_1)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.13."""
        _equation: str = r"\frac{\epsilon_1 + \epsilon_2}{2 \cdot \epsilon_1}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"\epsilon_1": f"{self.epsilon_1:.{n}f}",
                r"\epsilon_2": f"{self.epsilon_2:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"k_2",
            result=f"{self._evaluate(self.epsilon_1, self.epsilon_2):.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="-",
        )



"""Testing formula 6.31 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_31 import Form6Dot31CheckBendingAndAxialForce
from blueprints.validations import NegativeValueError


class TestForm6Dot31CheckBendingAndAxialForce:
    """Validation for formula 6.31 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        m_ed = 500.0
        m_n_rd = 600.0

        # Object to test
        formula = Form6Dot31CheckBendingAndAxialForce(m_ed=m_ed, m_n_rd=m_n_rd)

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    def test_evaluation2(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        m_ed = 700.0
        m_n_rd = 600.0

        # Object to test
        formula = Form6Dot31CheckBendingAndAxialForce(m_ed=m_ed, m_n_rd=m_n_rd)

        # Expected result, manually calculated
        expected_result = False

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("m_ed", "m_n_rd"),
        [
            (-500.0, 600.0),  # m_ed is negative
            (500.0, -600.0),  # m_n_rd is negative
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, m_ed: float, m_n_rd: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form6Dot31CheckBendingAndAxialForce(m_ed=m_ed, m_n_rd=m_n_rd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to M_{Ed} \leq M_{N,Rd} \to 500.000 \leq 600.000 \to OK",
            ),
            (
                "complete_with_units",
                r"CHECK \to M_{Ed} \leq M_{N,Rd} \to 500.000 \ Nmm \leq 600.000 \ Nmm \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        m_ed = 500.0
        m_n_rd = 600.0

        # Object to test
        latex = Form6Dot31CheckBendingAndAxialForce(m_ed=m_ed, m_n_rd=m_n_rd).latex()

        actual = {
            "complete": latex.complete,
            "complete_with_units": latex.complete_with_units,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 5.32 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_32 import Form5Dot32EquivalentFirstOrderEndMoment
from blueprints.validations import NegativeValueError


class TestForm5Dot32EquivalentFirstOrderEndMoment:
    """Validation for formula 5.32 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        m_01 = 30.0
        m_02 = 50.0

        # Object to test
        formula = Form5Dot32EquivalentFirstOrderEndMoment(m_01=m_01, m_02=m_02)

        # Expected result, manually calculated
        manually_calculated_result = 42.0  # kNm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("m_01", "m_02"),
        [
            (50.0, 30.0),  # m_02 is smaller than m_01
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, m_01: float, m_02: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, ValueError)):
            Form5Dot32EquivalentFirstOrderEndMoment(m_01=m_01, m_02=m_02)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"M_{0e} = \max\left(0.6 \cdot M_{02} + 0.4 \cdot M_{01}; 0.4 \cdot M_{02}\right) "
                r"= \max\left(0.6 \cdot 50.000 + 0.4 \cdot 30.000; 0.4 \cdot 50.000\right) = 42.000 \ kNm",
            ),
            ("short", r"M_{0e} = 42.000 \ kNm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        m_01 = 30.0
        m_02 = 50.0

        # Object to test
        latex = Form5Dot32EquivalentFirstOrderEndMoment(m_01=m_01, m_02=m_02).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 3.5 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_3_materials.formula_3_5 import Form3Dot5ApproximationVarianceElasticModulusOverTime


class TestForm3Dot5ApproximationVarianceElasticModulusOverTime:
    """Validation for formula 3.5 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        f_cm_t = 2.34  # MPa
        f_cm = 3.4  # MPa
        e_cm = 2.9  # MPa
        form_3_5 = Form3Dot5ApproximationVarianceElasticModulusOverTime(f_cm_t=f_cm_t, f_cm=f_cm, e_cm=e_cm)

        # Expected result, manually calculated
        manually_calculated_result = 2.592502

        assert form_3_5 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_f_cm_t_is_given(self) -> None:
        """Test a negative value."""
        # Example values
        f_cm_t = -2.34  # MPa
        f_cm = 3.4  # MPa
        e_cm = 2.9  # MPa

        with pytest.raises(ValueError):
            Form3Dot5ApproximationVarianceElasticModulusOverTime(f_cm_t=f_cm_t, f_cm=f_cm, e_cm=e_cm)

    def test_raise_error_when_negative_f_cm_is_given(self) -> None:
        """Test a negative value."""
        # Example values
        f_cm_t = 2.34  # MPa
        f_cm = -3.4  # MPa
        e_cm = 2.9  # MPa

        with pytest.raises(ValueError):
            Form3Dot5ApproximationVarianceElasticModulusOverTime(f_cm_t=f_cm_t, f_cm=f_cm, e_cm=e_cm)

    def test_raise_error_when_negative_e_cm_is_given(self) -> None:
        """Test a negative value."""
        # Example values
        f_cm_t = 2.34  # MPa
        f_cm = 3.4  # MPa
        e_cm = -2.9  # MPa

        with pytest.raises(ValueError):
            Form3Dot5ApproximationVarianceElasticModulusOverTime(f_cm_t=f_cm_t, f_cm=f_cm, e_cm=e_cm)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"E_{cm}(t) = ( f_{cm}(t) / f_{cm} )^{0.3} \cdot E_{cm} = ( 2.340 / 3.400 )^{0.3} \cdot 2.900 = 2.593",
            ),
            ("short", r"E_{cm}(t) = 2.593"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        f_cm_t = 2.34  # MPa
        f_cm = 3.4  # MPa
        e_cm = 2.9  # MPa

        # Object to test
        form_3_5_latex = Form3Dot5ApproximationVarianceElasticModulusOverTime(f_cm_t=f_cm_t, f_cm=f_cm, e_cm=e_cm).latex()

        actual = {"complete": form_3_5_latex.complete, "short": form_3_5_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Formula 6.47 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MPA
from blueprints.validations import raise_if_negative


class Form6Dot47PunchingShearResistance(Formula):
    r"""Class representing formula 6.47 for the calculation of punching shear resistance, $v_{Rd,c}$ of slabs and column bases
    without shear reinforcement.
    """

    label = "6.47"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        c_rd_c: DIMENSIONLESS,
        k: DIMENSIONLESS,
        rho_l: DIMENSIONLESS,
        f_ck: MPA,
        k_1: DIMENSIONLESS,
        sigma_cp: MPA,
        v_min: MPA,
    ) -> None:
        r"""$v_{Rd,c}$ Calculation of punching shear resistance of slabs and column bases without shear reinforcement.

        EN 1992-1-1:2004 art.6.4.4(1) - Formula (6.47).

        The values of $C_{Rd,c}$, $v_{min}$, and $k_1$ for use in a country may be found in its national annex.
        The recommended value for $C_{Rd,c}$ is $0.18 / \gamma_c$, for $v_{min}$ is given by Expression (6.3N),
        and that for $k_1$ is $0.1$.

        Parameters
        ----------
        c_rd_c : DIMENSIONLESS
            $C_{Rd,c}$ Coefficient for punching shear resistance, recommended value $0.18 / \gamma_c$ [-].
        k : DIMENSIONLESS
            $k$ Size effect factor, see equation SubForm6Dot47FactorK [-].
        rho_l : DIMENSIONLESS
            $\rho_l$ Longitudinal reinforcement ratio, see equation SubForm6Dot47FactorRhoL [-].
        f_ck : MPA
            $f_{ck}$ Characteristic compressive strength of concrete [$MPa$].
        k_1 : DIMENSIONLESS
            $k_1$ Coefficient for concrete strength, recommended value 0.1 [-].
        sigma_cp : MPA
            $\sigma_{cp}$ Stress in the critical section as average of the two perpendicular directions, see
             equation SubForm6Dot47FactorSigmaCp [$MPa$].
        v_min : MPA
            $v_{min}$ Minimum shear stress capacity concrete, recommended value with Expression (6.3N) [$MPa$].
        """
        super().__init__()
        self.c_rd_c = c_rd_c
        self.k = k
        self.rho_l = rho_l
        self.f_ck = f_ck
        self.k_1 = k_1
        self.sigma_cp = sigma_cp
        self.v_min = v_min

    @staticmethod
    def _evaluate(
        c_rd_c: DIMENSIONLESS,
        k: DIMENSIONLESS,
        rho_l: DIMENSIONLESS,
        f_ck: MPA,
        k_1: DIMENSIONLESS,
        sigma_cp: MPA,
        v_min: MPA,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(c_rd_c=c_rd_c, k=k, rho_l=rho_l, f_ck=f_ck, k_1=k_1, sigma_cp=sigma_cp, v_min=v_min)

        term1 = c_rd_c * k * (100 * rho_l * f_ck) ** (1 / 3) + k_1 * sigma_cp
        term2 = v_min + k_1 * sigma_cp
        return max(term1, term2)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.47."""
        _equation: str = (
            r"\max \left( C_{Rd,c} \cdot k \cdot (100 \cdot \rho_l \cdot f_{ck})^{1/3} "
            r"+ k_1 \cdot \sigma_{cp}, v_{min} + k_1 \cdot \sigma_{cp} \right)"
        )
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"C_{Rd,c}": f"{self.c_rd_c:.{n}f}",
                r"\rho_l": f"{self.rho_l:.{n}f}",
                r"f_{ck}": f"{self.f_ck:.{n}f}",
                r"k_1": f"{self.k_1:.{n}f}",
                r"\sigma_{cp}": f"{self.sigma_cp:.{n}f}",
                r"v_{min}": f"{self.v_min:.{n}f}",
                r"k": f"{self.k:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"v_{Rd,c}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="MPa",
        )



"""Formula 7.4 from EN 1992-1-1:2004: Chapter 7 - Serviceability Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import MM, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form7Dot4MeanStressConcrete(Formula):
    r"""Class representing formula 7.4 for the calculation of [$\sigma_c$]."""

    label = "7.4"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        n_ed: N,
        b: MM,
        h: MM,
    ) -> None:
        r"""[$\sigma_c$] Calculation of mean stress of the concrete in the cross-section under consideration [$MPa$].

        EN 1992-1-1:2004 art.7.3.2(2) - Formula (7.4)

        Parameters
        ----------
        n_ed : N
            [$N_{Ed}$] Axial force at the serviceability limit state acting on the part of the
            cross-section under consideration (compressive force positive) [$N$].
        b : MM
            [$b$] Width of the cross-section [$mm$].
        h : MM
            [$h$] Height of the cross-section [$mm$].
        """
        super().__init__()
        self.n_ed = n_ed
        self.b = b
        self.h = h

    @staticmethod
    def _evaluate(
        n_ed: N,
        b: MM,
        h: MM,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(b=b, h=h)
        raise_if_negative(n_ed=n_ed)

        return n_ed / (b * h)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.4."""
        _equation: str = r"\frac{N_{Ed}}{b \cdot h}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"N_{Ed}": f"{self.n_ed:.{n}f}",
                r"b": f"{self.b:.{n}f}",
                r"h": f"{self.h:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"\sigma_c",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="MPa",
        )



"""Formula 12.3 from EN 1992-1-1:2004: Chapter 12 - Plain and Lightly Reinforced Concrete Structures."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM2, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form12Dot3PlainConcreteShearStress(Formula):
    r"""Class representing formula 12.3 for the calculation of the design shear stress of plain concrete,
    :math:`\sigma_{cp}`.

    EN 1992-1-1:2004 art.12.6.3(2) - Formula (12.3)
    """

    label = "12.3"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        n_ed: N,
        a_cc: MM2,
    ) -> None:
        r"""[:math:`\sigma_{cp}`] Design shear stress of plain concrete [:math:`MPa`].

        EN 1992-1-1:2004 art.12.6.3(2) - Formula (12.3)

        Parameters
        ----------
        n_ed : N
            [:math:`N_{Ed}`] Design normal force [:math:`N`].
        a_cc : MM2
            [:math:`A_{cc}`] Compressed area [:math:`mm^2`].
        """
        super().__init__()
        self.n_ed = n_ed
        self.a_cc = a_cc

    @staticmethod
    def _evaluate(
        n_ed: N,
        a_cc: MM2,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(
            a_cc=a_cc,
        )

        raise_if_negative(
            n_ed=n_ed,
        )

        return n_ed / a_cc

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 12.3."""
        return LatexFormula(
            return_symbol=r"\sigma_{cp}",
            result=f"{self:.{n}f}",
            equation=r"\frac{N_{Ed}}{A_{cc}}",
            numeric_equation=rf"\frac{{{self.n_ed:.{n}f}}}{{{self.a_cc:.{n}f}}}",
            comparison_operator_label="=",
        )



"""Tests for the StripClass enum."""

from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.strip import Strip


class TestStripClass:
    """Tests for the StripClass enum."""

    def test_enum_values(self) -> None:
        """Test that enum values are correctly defined."""
        assert Strip.STRIP160x5.value == ("160x5", 160, 5)
        assert Strip.STRIP230x25.value == ("230x25", 230, 25)

    def test_enum_membership(self) -> None:
        """Test that specific values are members of the enum."""
        assert "160x5" in [e.value[0] for e in Strip]
        assert "230x25" in [e.value[0] for e in Strip]

    def test_enum_uniqueness(self) -> None:
        """Test that all enum values are unique."""
        values = [e.value for e in Strip]
        assert len(values) == len(set(values))

    def test_enum_attributes(self) -> None:
        """Test that enum attributes are correctly assigned."""
        profile = Strip.STRIP160x5
        assert profile.alias == "160x5"
        assert profile.width == 160
        assert profile.height == 5



"""Formula 5.5 from EN 1993-5:2007 Chapter 5 - Ultimate limit states."""

import numpy as np

from blueprints.codes.eurocode.en_1993_5_2007 import EN_1993_5_2007
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_fraction
from blueprints.type_alias import DIMENSIONLESS, KN, MM2, MPA
from blueprints.unit_conversion import N_TO_KN
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form5Dot5PlasticShearResistance(Formula):
    """Class representing formula 5.5 for the design plastic shear resistance for each web."""

    label = "5.5"
    source_document = EN_1993_5_2007

    def __init__(
        self,
        a_v: MM2,
        f_y: MPA,
        gamma_m_0: DIMENSIONLESS,
    ) -> None:
        r"""([$V_{pl,Rd}$]) Calculate design plastic shear resistance for each web in [$kN$].

        EN 1993-5:2007(E) art.5.2.2(4) - Formula (5.5)

        Parameters
        ----------
        a_v : MM2
            ([$A_{v}$]) Projected shear area for each web, acting in the same direction as VEd in [$mm^2$].
        f_y : MPA
            ([$f_{y}$]) Yield strength in [$MPa$].
        gamma_m_0 : DIMENSIONLESS
            ([$\gamma_{M0}$]) Partial factor for material properties in [$-$].
        """
        super().__init__()
        self.a_v: float = a_v
        self.f_y: float = f_y
        self.gamma_m_0: DIMENSIONLESS = gamma_m_0

    @staticmethod
    def _evaluate(
        a_v: MM2,
        f_y: MPA,
        gamma_m_0: DIMENSIONLESS,
    ) -> KN:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(a_v=a_v, f_y=f_y, gamma_m_0=gamma_m_0)
        return (a_v * f_y / (np.sqrt(3) * gamma_m_0)) * N_TO_KN

    def latex(self, n: int = 2) -> LatexFormula:
        """Returns LatexFormula object for formula 5.5."""
        return LatexFormula(
            return_symbol=r"V_{pl,Rd}",
            result=f"{self:.{n}f}",
            equation=latex_fraction(
                numerator="A_v f_y",
                denominator=r"\sqrt{3} \gamma_{M0}",
            ),
            numeric_equation=latex_fraction(
                numerator=rf"{self.a_v} \cdot {self.f_y}",
                denominator=rf"\sqrt{{3}} \cdot {self.gamma_m_0}",
            ),
            comparison_operator_label="=",
        )
